import config
import measfcts
import os
import numpy as np
import matplotlib.ticker as ticker
import matplotlib

import tenbilac
import momentsml.plot
from momentsml.tools.feature import Feature
import matplotlib.pyplot as plt

import logging
logger = logging.getLogger(__name__)


#momentsml.plot.figures.set_fancy(14)
from matplotlib import rc
rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
## for Palatino and other serif fonts use:
#rc('font',**{'family':'serif','serif':['Palatino']})
rc('text', usetex=True)


workbasedir = os.path.join(config.traindir, config.datasets["tp"])


for (dataconfpath, toolconfpath) in config.shearconflist:

	toolconfig = momentsml.learn.tenbilacrun.readconfig(toolconfpath)
	dataconfig = momentsml.learn.tenbilacrun.readconfig(dataconfpath)
	confname = dataconfig.get("setup", "name") + "_" + toolconfig.get("setup", "name")
	trainworkdir = os.path.join(workbasedir, confname) # We will pass this to Tenbilac
	
	
	tblconfiglist = [("setup", "workdir", trainworkdir)]
	ten = tenbilac.com.Tenbilac(toolconfpath, tblconfiglist)
       
	ten._readmembers()
	
	
	fig = plt.figure(figsize=(6, 3.5))

	ax = fig.add_subplot(1, 1, 1)

	tenbilac.plot.summaryerrevo(ten.committee, ax=ax)       
	
	ax.set_yscale('log')
	ax.set_xscale('log')
	ax.set_xlabel(r"Iteration")
	ax.set_ylim(1e-6, 1e-3)
	ax.set_xlim((5, 1e4))
	ax.set_ylabel(r"MSB cost function value")
	#ax.legend()

	plt.tight_layout()


	momentsml.plot.figures.savefig(os.path.join(config.valdir, config.datasets["ts"] + "_" + confname + "_msbevo"), fig, fancy=True, pdf_transparence=True)
	plt.show()



"""
Shape measurement with GalSim's adaptive moments (from its "hsm" module).
"""

import numpy as np
import sys, os
import copy
from datetime import datetime

import logging
logger = logging.getLogger(__name__)

import astropy.table
import galsim

import utils
from .. import tools

try:
	import scipy.ndimage
except:
	logger.warning("Could not import scipy.ndimage, adamom_size not available")


def measfct(catalog, runon="img", stampsize=None, **kwargs):
	"""
	This is a wrapper around galsim_adamom that meets the requirements of a MomentsML-conform shape measurement function, namely
	to take only one catalog (astropy table) object containing -- or link to -- all the required data.
	In other words, this is a function that you could pass to meas.run.general() etc.
	If you want to combine several shape measurement algorithms into one shot, you would define such a function yourself (not here
	in momentsml, but somewhere in your scripts).
	The present measfct serves as an example and is a bit long. It could be kept very short.
	
	:param catalog: an astropy table, which, in this case, is expected to have catalog.meta["img"] set
		to be a momentsml.tools.imageinfo.ImageInfo object.
	:param runon: "img" or "psf" or other ImageInfo names to run on.
	:param kwargs: keyword arguments that will be passed to the lower-level measure() function.
		These set parameters of the shape measurement, but they do not pass any data.
		Do not try to specify "img" or "xname" here, it will fail! Set the catalog's meta["img"] instead.
		So for this particular measfct, you probably want to give at least stampsize as kwarg.
	
	"""
	
	# We get the stamp size to use:
	stampsize = catalog.meta[runon].get_stampsize(stampsize)
	
	# We load the image:
	img = catalog.meta[runon].load()

	# And we pass it, with all required kwargs, to the lower-level function:
	return measure(img, catalog, xname=catalog.meta[runon].xname, yname=catalog.meta[runon].yname, stampsize=stampsize, **kwargs)


def measure(img, catalog, xname="x", yname="y", stampsize=None, prefix="adamom_", variant="default", size=False):
	"""
	Use the pixel positions provided via the 'catalog' input table to extract
	postage stamps from the image and measure their shape parameters.
	Returns a copy of your input catalog with the new (masked) columns appended.
	One of these colums (the only one that is not masked) is the flag:
	
	* 0: OK
	* 1: stamp is not fully within image
	* 2: galsim centroid failed (distance > 10 pixels...)
	* 3: galsim failed
	
	:param img: either the path to a FITS image, or a galsim image object
	:param catalog: astropy table of objects to be measured
	:param xname: column name containing the x coordinates in pixels
	:param yname: idem for y
	:param stampsize: width = height of stamps, has to be even
	:type stampsize: int
	:param prefix: a string to prefix the field names that I'll write
	:param variant: a switch for different variants of parameters for this method. 'default' uses defaults,
		'wider' starts failed default-measurements again with a larger sigma and more iterations.
	:type variant: string
	
	
	:returns: masked astropy table
	
	"""
	
	if type(img) is str:
		logger.debug("You gave me a filepath, and I'm now loading the image...")
		img = tools.image.loadimg(img)
	
	if int(stampsize)%2 != 0:
		raise RuntimeError("The stampsize should be even!")

	starttime = datetime.now()
	logger.info("Starting measurements on %ix%i stamps, with variant='%s'" % (stampsize, stampsize, variant))
	
	# We prepare an output table with all the required columns
	output = astropy.table.Table(copy.deepcopy(catalog), masked=True) # Convert the table to a masked table
	# A bit strange: reading the doc I feel that this conversion is not needed.
	# But without it, it just doesn't result in a masked table once the masked columns are appended.

	output.add_columns([
		astropy.table.Column(name=prefix+"flag", data=np.zeros(len(output), dtype=int)), # We will always have a flag
		astropy.table.MaskedColumn(name=prefix+"flux", dtype=float, length=len(output)),
		astropy.table.MaskedColumn(name=prefix+"x", dtype=float, length=len(output)),
		astropy.table.MaskedColumn(name=prefix+"y", dtype=float, length=len(output)),
		astropy.table.MaskedColumn(name=prefix+"g1", dtype=float, length=len(output)),
		astropy.table.MaskedColumn(name=prefix+"g2", dtype=float, length=len(output)),
		astropy.table.MaskedColumn(name=prefix+"sigma", dtype=float, length=len(output)),
		astropy.table.MaskedColumn(name=prefix+"rho4", dtype=float, length=len(output)),
		astropy.table.MaskedColumn(name=prefix+"size", dtype=float, length=len(output))
	])
	
	# We want to mask all these entries. They will get unmasked when values will be attributed.
	for col in ["flux", "x", "y", "g1", "g2", "sigma", "rho4"]:
		output[prefix+col].mask = [True] * len(output) # "True" means masked !
		
	n = len(output)
	
	# And loop
	for gal in output:
		
		# Some simplistic progress indication:
		if gal.index%5000 == 0:
			logger.info("%6.2f%% done (%i/%i) " % (100.0*float(gal.index)/float(n), gal.index, n))
		
		(x, y) = (gal[xname], gal[yname])
		(gps, flag) = tools.image.getstamp(x, y, img, stampsize)
		
		if flag != 0:
			logger.debug("Galaxy not fully within image:\n %s" % (str(gal)))
			gal[prefix+"flag"] = flag
			# We can't do anything, and we just skip this galaxy.
			continue
				
		# And now we measure the moments... galsim may fail from time to time, hence the try:
		if variant == "default":
			try: # We simply try defaults:
				res = galsim.hsm.FindAdaptiveMom(gps)
			except:
				# This is awesome, but clutters the output 
				#logger.exception("GalSim failed on: %s" % (str(gal)))
				# So instead of logging this as an exception, we use debug, but include the traceback :
				logger.debug("HSM with default settings failed on:\n %s" % (str(gal)), exc_info = True)	
				gal[prefix + "flag"] = 3	
				continue # skip to next stamp !
		
		elif variant == "wider":
		
			try:
				try: # First we try defaults:
					res = galsim.hsm.FindAdaptiveMom(gps)
				except: # We change a bit the settings:
					logger.debug("HSM defaults failed, retrying with larger sigma...")
					hsmparams = galsim.hsm.HSMParams(max_mom2_iter=1000)
					res = galsim.hsm.FindAdaptiveMom(gps, guess_sig=15.0, hsmparams=hsmparams)			

			except: # If this also fails, we give up:
				logger.debug("Even the retry failed on:\n %s" % (str(gal)), exc_info = True)	
				gal[prefix + "flag"] = 3	
				continue
		
		else:
			raise RuntimeError("Unknown variant setting '{variant}'!".format(variant=variant))
	
		gal[prefix+"flux"] = res.moments_amp
		gal[prefix+"x"] = res.moments_centroid.x + 1.0 # Not fully clear why this +1 is needed. Maybe it's the setOrigin(0, 0).
		gal[prefix+"y"] = res.moments_centroid.y + 1.0 # But I would expect that GalSim would internally keep track of these origin issues.
		gal[prefix+"g1"] = res.observed_shape.g1
		gal[prefix+"g2"] = res.observed_shape.g2
		gal[prefix+"sigma"] = res.moments_sigma
		gal[prefix+"rho4"] = res.moments_rho4

		if size:
			imgstamp = gps.array
			
			facta = 4.
			try:
				imgstamp = scipy.ndimage.zoom(imgstamp, facta, order=0) / facta / facta
			except:
				facta = 1.
	
			dx = np.linspace(0, 48, 48*facta)
			dy = np.linspace(0, 48, 48*facta)[:,None]
			X = dx - (res.moments_centroid.x - x + 25.)
			Y = dy - (res.moments_centroid.y - y + 25.)
			a, b, theta = tools.calc.complex2geometrical(res.observed_shape.g1, res.observed_shape.g2)
			e = a / b 
			err = 1.
			
			# The factor is empirical only
			a = res.moments_sigma/np.sqrt(e) * 2.8
			b = a / e
			
			fact = 0
			ccc = 0
			while err > 0.05 and ccc < 100:
				a += fact
				b = a / e
				ellipse = (X * np.cos(theta) + Y * np.sin(theta)) ** 2. / (a*a) + (Y * np.cos(theta) - X * np.sin(theta)) ** 2. / (b*b) <= 1
				F = imgstamp * ellipse
			
				#print F.sum()
				#print F.sum() / res.moments_amp
				
				fact = gal["adamom_flux"] - F.sum()
				
				err = np.abs(F.sum() - res.moments_amp) / res.moments_amp
				ccc += 1
				
			gal[prefix+"size"] = a * b * np.pi
			#plt.figure()
			#plt.imshow(ellipse * imgstamp, extent=(dx[0], dx[-1], dy[0], dy[-1]), origin="lower", interpolation="None")
			#plt.show()

		else:
			gal[prefix+"size"] = 0.

		# If we made it to this point, we check that the centroid is roughly ok:
		if np.hypot(x - gal[prefix+"x"], y - gal[prefix+"y"]) > 10.0:
			gal[prefix + "flag"] = 2
			
		if gal[prefix+"flux"] < 0 and gal[prefix + "flag"] > 0:
			# The centroid checking is rough
			gal[prefix + "flag"] = 4
		

	endtime = datetime.now()	
	logger.info("All done")

	nfailed = np.sum(output[prefix+"flag"] > 0)
	
	logger.info("I failed on %i out of %i sources (%.1f percent)" % (nfailed, n, 100.0*float(nfailed)/float(n)))
	logger.info("This measurement took %.3f ms per galaxy" % (1e3*(endtime - starttime).total_seconds() / float(n)))

	return output



"""
Helpers for astropy.table arrays
"""

import numpy as np
import astropy.table
import datetime

from . import calc

import copy

import logging
logger = logging.getLogger(__name__)


def info(cat, txt=True):
	"""
	Returns a new table "describing" the content of the table cat.
	"""
	colnames = cat.colnames
	dtypes = [cat[colname].dtype for colname in colnames]
	ndims = [cat[colname].ndim for colname in colnames]
	shapes = [cat[colname].shape for colname in colnames]
	infotable = astropy.table.Table([colnames, dtypes, ndims, shapes], names=("colname", "dtype", "ndim", "shape"))
	
	infotable.sort("colname")
	infotable.meta = cat.meta
	
	if txt:
		
		lines = infotable.pformat(max_lines=-1, max_width=-1)
		lines.append("")
		lines.append("Number of rows: {}".format(len(cat)))
		lines.append("Number of columns: {}".format(len(cat.colnames)))
		lines.append("Metadata: {}".format(str(infotable.meta.items())))
		
		return "\n".join(lines)
	else:
		return infotable





def hjoin(table1, table2, idcol):
	"""
	This is a safe way to "hstack" two astropy tables containing different columns
	for the same sources, using a specified "ID" column.
	
	These kind of things are easy to do with astropy tables.
	The present function is just a wrapper for one type of operation that is very
	frequently required when running MomentsML.

	Instead of a simple hstack (which would rely on the order of rows), here we use 
	join to match the rows, using the idcol (i.e., "ID") column.
	We guarantee that the ouput table will have the same number of
	rows as the input tables.
	
	This function could be made more general/powerful (as join is much more general),
	but it is kept simple on purpose.
	
	:param table1: an astropy table
	:param table2: an astropy table
	:param idcol: a column label, to be used as key for the identification
	              (must exist in both tables)
	
	If there are columns that have the same label in both table1 and table2, the column values
	will be taken from table1, so as to avoid duplicated columns in the resulting AstroPy table.
	
	"""

	logger.debug("Starting some security checks (%i rows)..." % len(table1))
	
	if len(table1) != len(table2):
		raise RuntimeError("Your tables do not have the same row counts!")
	
	
	if idcol not in table1.colnames:
		raise RuntimeError("There is no ID column '%s' in table1" % (idcol))
	if idcol not in table2.colnames:
		raise RuntimeError("There is no ID column '%s' in table2" % (idcol))
		
	set1 = set(table1[idcol])
	set2 = set(table2[idcol])
	if len(set1) != len(table1):
		raise RuntimeError("The IDs in table1 are not unique!")
	if len(set2) != len(table2):
		raise RuntimeError("The IDs in table2 are not unique!")
	if set1 != set2:
		raise RuntimeError("The IDs in the two tables are not identical!")
		
	# Now we find out which cols to keep in table2 when calling join
	# (without changing table2 of course).
	
	commoncols = [col for col in table1.colnames if col in table2.colnames]
	if len(commoncols) > 1: # Otherwise it's just the idcol...
		logger.info("The column labels %s are common, values from table1 will be kept." % (str(commoncols)))
	table2colstokeep = [col for col in table2.colnames if col not in commoncols]
	table2colstokeep.append(idcol)
	
	# Indeed, join would duplicate and rename common columns (even if they contain the same values)
	# Unless keys = None, in which case it would use them to identify.
	# But then, if those cols are different, the identification fails and join would silently keep
	# the "left" data, with masked values for the columns of table2 (as if "not found")
	# That's not what we want.
	# What we want is to raise an Error if the idcol values are fishy.
	
	logger.debug("Done with security checks")
	
	output = astropy.table.join(table1, table2[table2colstokeep], keys=idcol,
		join_type='left', metadata_conflicts="error")

	assert len(output) == len(table1)
	assert len(output.colnames) == len(table1.colnames) + len(table2colstokeep) - 1
	
	return output
	

def shuffle(table):
	"""
	Return a copy of the input table with randomly shuffled rows.
	Easy function, but it demonstrates one way of how this can be done.
	"""
	
	logger.debug("Shuffling a table of %i rows..." % len(table))
	
	indexes = np.arange(len(table))
	np.random.shuffle(indexes)
	return table[indexes] # This is a copy
	





def group(incats, groupcols=None, removecols=None, checkcommon=True):
	"""
	Turns a list of catalogs (typically representing different realizations of the same galaxies) into a single "3D" catalog.
	This is somehow similar to groupstats, but here we do not compute any stats, and we don't modify column names.
		
	Returns an astropy table in which the groupcols columns are 2D arrays.
	
	This is easier to digest for Tenbilac etc than to carry around many columns _rea0, _rea1, ...
	
	"""
	starttime = datetime.datetime.now()
	
	if groupcols is None:
		groupcols = []
	if removecols is None:
		removecols = [] 

	# OK I'm taking here a few simple tests from from groupstats, as they are quite helpful.

	colnames = incats[0].colnames # to check colnames
	
	for incat in incats:
		if incat.colnames != colnames:
			raise RuntimeError("Your input catalogs do not have the same columns: \n\n %s \n\n is not \n\n %s"
				% (incat.colnames, colnames))

	for groupcol in groupcols:
		if groupcol not in colnames:
			raise RuntimeError("The column '%s' which should be grouped is not present in the catalog. The availble column names are %s" % (groupcol, colnames))
		if groupcol in removecols:
			raise RuntimeError("Cannot both group and remove column '%s' " % groupcol)


	# We make a list of the column names that should stay unaffected:
	fixedcolnames = [colname for colname in colnames if (colname not in groupcols) and (colname not in removecols)]
	
	if len(fixedcolnames) > 0:
		# We will take those columns from the first incat (we test below that this choice doesn't matter)
		fixedcat = incats[0][fixedcolnames] # This makes a copy
	
		if checkcommon == True:
			# We test that the columns of these fixedcolnames are the same for all the different incats
			for incat in incats:
				if not np.all(incat[fixedcolnames] == fixedcat):
					print incat.colnames
					print fixedcat.colnames
					print incat[fixedcolnames].colnames 
					print fixedcat
					raise RuntimeError("Something fishy is going on: some columns are not identical among %s. Add them to groupcols or removecols." % fixedcolnames)
			logger.debug("Done with testing the identity of all the common columns")
		else:
			logger.debug("Did not test the identity of all the common columns")

	outputcols = [] # We fill this with the new 2D columns...
	for groupcol in groupcols:
		#logger.info("Grouping '%s'" % (groupcol))
		
		# We build a masked numpy array with the data of these columns
		array = np.ma.vstack([incat[groupcol] for incat in incats]).transpose()
		
		#print array.shape # With this transpose, first index is galaxy, second is realization

		outputcols.append(astropy.table.MaskedColumn(array, name=groupcol))

	
	# We now make a table out of the outputcols :
	outputcat = astropy.table.Table(outputcols)
	
	# Finally, we prepend the fixedcolname-columns to the table:
	if len(fixedcolnames) > 0:
		outputcat = astropy.table.hstack([fixedcat, outputcat], join_type="exact",
			table_names=["SHOULD_NOT_BE_SEEN", "SHOULD_NEVER_BE_SEEN"], uniq_col_name="{col_name}_{table_name}", metadata_conflicts="error")
	
	# Especially when averaging measurements on realizations, it seems ok to write ngroupstats also into the meta of the catalog itself.
	# So we keep doing this.
	outputcat.meta["ngroup"] = len(incats)

	endtime = datetime.datetime.now()
	logger.info("The grouping took %s" % (str(endtime - starttime)))
	logger.info("Output table: %i rows and %i columns (%i common, %i grouped)" %
		(len(outputcat), len(outputcat.colnames), len(fixedcolnames), len(outputcols)))
	
	return outputcat



	
def groupstats(incats, groupcols=None, removecols=None, removereas=True, keepfirstrea=True, checkcommon=True):
	"""
	This function computes simple statistics "across" corresponding columns from the list of input catalogs (incats).
	Instead of producing 2D columns (i.e., a 3D table), it puts all the data into extra columns with custom names ("_rea0"...).
	
	:param incats: list of input catalogs (astropy tables, usually masked).
		They must all have identical order and column names (this will be checked).
	:param groupcols: list of column names that should be "grouped", that is the columns whose content differ from incat to incat.
	:param removecols: list of column names that should be discarded in the output catalog
	:param removereas: if True, the individual realization columns in the output table will **not** be kept in the ouput
		(except maybe the first one, see keepfirstreas).
		Setting this to False can result in very bulky catalogs.
	:param keepfirstreas: if True, the values of the first realization ("_0") will be kept.
		It is usually handy (and not too bulky) to "keep" one single realization in the output catalog.
	:param checkcommon: if True (default), the function tests that **any column which is not in groupcols or removecols is indeed IDENTICAL among the incats**.
		This can lead to perfomance issues for catalogs with many (say > 1000) columns.
	
	For each colname in groupcols, the function computes:
	
	- **colname_mean**: the average of colname from the different incats (skipping masked measurements)
	- **colname_med**: the median (idem)
	- **colname_std**: the sample standard deviation (idem)
	- **colname_n**: the number of available (that is, unmasked) measurements
	
	The function returns a single catalog, and the above column names designate the new columns that will be added to this catalog.
	
	.. note:: The number of incats (which is the maximum possible value for "colname_n") is stored as meta["ngroupstats"]
		of each of the newly created columns.	
		When running on measurements obtained on different realizations, this "ngroupstats" allows for instance to compute
		the success fraction of the measurement:  outcat["foo_n"] / outcat["foo_n"].meta["ngroupstats"]  
		
	Developer note: it is slow to append columns to astropy tables (as this makes a copy of the full table).
		So we try to avoid this as much as possible here, and use masked numpy arrays and hstack.
	
	"""
	
	starttime = datetime.datetime.now()
	
	if groupcols is None:
		groupcols = []
	if removecols is None:
		removecols = [] 

	# First, some checks on the incats:
	if len(incats) < 2:
		raise RuntimeError("Statistics can only be computed if more than one incats are given.")
	
	colnames = incats[0].colnames # to check colnames
	
	notmasked = False
	for incat in incats:
		if incat.colnames != colnames:
			raise RuntimeError("Your input catalogs do not have the same columns: \n\n %s \n\n is not \n\n %s"
				% (incat.colnames, colnames))
		
		if incat.masked is False:
			notmasked = True
	
	if notmasked:
		logger.info("At least one of the input catalogs is not masked (OK but unexpected)")

	for groupcol in groupcols:
		if groupcol not in colnames:
			raise RuntimeError("The column '%s' which should be grouped is not present in the catalog. The availble column names are %s" % (groupcol, colnames))
		if groupcol in removecols:
			raise RuntimeError("Cannot both group and remove column '%s' " % groupcol)
			
	# We make a list of the column names that should stay unaffected:
	fixedcolnames = [colname for colname in colnames if (colname not in groupcols) and (colname not in removecols)]
	
	if len(fixedcolnames) > 0:
		# We will take those columns from the first incat (we test below that this choice doesn't matter)
		fixedcat = incats[0][fixedcolnames] # This makes a copy
	
		if checkcommon == True:
			# We test that the columns of these fixedcolnames are the same for all the different incats
			for incat in incats:
				if not np.all(incat[fixedcolnames] == fixedcat):
					raise RuntimeError("groupstat failed: colnames match, but some columns are not \
identical among the list. A colname is surely missing from to groupcols or removecols.")
			logger.debug("Done with testing the identity of all the common columns")
		else:
			logger.debug("Did not test the identity of all the common columns")
	
	# We prepare some "suffixes" to use when mixing columns of the incats.
	# For this we do not try to reuse the int from the realization filename
	# Indeed, the user could have deleted some realizations etc, leading to quite a mess.
	# It's easier to just make a new integer range:
	incat_names = ["rea%i" % (i) for i in range(len(incats))]

	statscatdict = {} # We will add statistics columns (numpy arrays) to this list
	statscatdictnames = [] # Is used to keep a nice ordering
	reascats = [] # We might put single-realization columns here
	
	# For each groupcol, we now compute some statistics
	for groupcol in groupcols:
		logger.info("Computing stats for '%s'" % (groupcol))
		suffixedcolnames = ["%s_%s" % (groupcol, incat_name) for incat_name in incat_names]
		# So this looks like adamom_flux_1, adamom_flux_2, ...
		
		# We build a masked numpy array with the data of these columns
		array = np.ma.vstack([incat[groupcol] for incat in incats])
		# first index is incat, second is row
		
		# And compute the stats:
		statscatdict["%s_mean" % (groupcol)] = np.ma.mean(array, axis=0)
		statscatdict["%s_med" % (groupcol)] = np.ma.median(array, axis=0)
		statscatdict["%s_std" % (groupcol)] = np.ma.std(array, axis=0)
		statscatdict["%s_n" % (groupcol)] = np.ma.count(array, axis=0)
		# We also add those names to a list:
		statscatdictnames.extend(["%s_mean" % (groupcol), "%s_med" % (groupcol), "%s_std" % (groupcol), "%s_n" % (groupcol)])
		
		# Depening on the removereas and keepfirstreas flags, we also keep some columns
		# from the individual realizations.
		if removereas == False:
			# We keep every column:
			reascats.append(astropy.table.Table([incat[groupcol] for incat in incats], names = suffixedcolnames))
			 
		else:
			# see if we should keep the first:
			if keepfirstrea == True:
				# keep the first
				reascats.append(astropy.table.Table([incat[groupcol] for incat in [incats[0]]], names = [suffixedcolnames[0]]))
			
	assert len(statscatdict) == 4*len(groupcols) # Just a check, as in principle stuff in the dict could be overwritten.

	# We now make a table out of the statscatdict :
	statscat = astropy.table.Table(statscatdict, names=statscatdictnames)
	
	# We add the number of catalogs that got grouped in the meta of every new "groupstats" column.
	# Strictly speaking this is where this information belongs, as it relates to the new columns, not to the catalog.
	# See note a few lines below.
	for colname in statscat.colnames:
		statscat[colname].meta["ngroupstats"] = len(incats)
	
	# We add individual realization data (if needed) :
	if len(reascats) != 0:
		statscat = astropy.table.hstack([statscat] + reascats, join_type="exact",
		metadata_conflicts="error")
	
	# Finally, we prepend the fixedcolname-columns to the table:
	if len(fixedcolnames) > 0:
		outputcat = astropy.table.hstack([fixedcat, statscat], join_type="exact",
			table_names=["SHOULD_NOT_BE_SEEN", "SHOULD_NEVER_BE_SEEN"], uniq_col_name="{col_name}_{table_name}", metadata_conflicts="error")
	else:
		outputcat = statscat
	
	# Especially when averaging measurements on realizations, it seems ok to write ngroupstats also into the meta of the catalog itself.
	# So we keep doing this.
	outputcat.meta["ngroupstats"] = len(incats)

	endtime = datetime.datetime.now()
	logger.info("The groupstats computations took %s" % (str(endtime - starttime)))
	logger.info("Output table: %i rows and %i columns (%i common, %i computed, %i reas)" %
		(len(outputcat), len(outputcat.colnames), len(fixedcolnames), len(statscatdict), len(statscat.colnames)-len(statscatdict)))
	
	return outputcat




def collapse(cat):
	"""
	Transforms a table into a new table with a single row, all the data being flattenend into the second dimension of each column.
	"""
	
	cols = []
	for colname in cat.colnames:
		
		data = np.ma.array(cat[colname]).flatten()
		assert data.ndim == 1
		data = data.reshape((1, data.size))
		cols.append(data)
		#print data.shape
	
	return astropy.table.Table(cols, names=cat.colnames)
	

def keepunique(cat, colnames=None, skipuniquetest=False):
	"""
	For each col in colnames, if the column is 2D (and the contents of each cell is identical),
	makes the column 1D by keeping only one of the identical values in each cell.
	If colnames is not specified, this is applied on all columns.
	
	Works in place, does not return a new cat!
	
	Note the make2D func below
	"""
	
	
	if colnames == None:
		proccolnames = cat.colnames # We will just run on all of them
		logger.info("Keepuniquing all {} columns (if possible)...".format(len(proccolnames)))
		if skipuniquetest:
			logger.warning("Caution: keepuniquing all columns without checking if the values are actually unique!")
	else:
		proccolnames = colnames
		
	for colname in proccolnames:
		
		#if colnames != None:
		#	logger.info("Keepuniquing column '{}'...".format(colname))
		
		if cat[colname].ndim == 2:
			
			#if skipuniquetest: # Then we do not check the cell contents, just pick the first elements:
			#else: # we do check the cell contents for uniqueness
			
			aresame = True
			if skipuniquetest == False:
				firstvals = cat[colname][:,0].data # All rows, but just first element of the second index
				tests = cat[colname].data.transpose() == firstvals			
				if not np.all(tests):
					aresame = False
			
			if aresame == True: # We just keep the first element in each cell:
				logger.info("Keepuniquing column '{}', original shape is {}...".format(colname, cat[colname].shape))
				newcol = cat[colname][:,0]
				cat.remove_column(colname)
				cat[colname] = newcol # Is a bit weird, but otherwise it complains about non-matching shapes
				
			else: # We do not keepunique it, but warn the user
				if colnames != None: # So if the user did explicitly specify colnames:
					logger.warning("Cells of column '{}' do not contain identical values, cannot keepunique it!".format(colname))
	
		else:
			if colnames != None: # So if the user did explicitly specify colnames:
				logger.warning("Column '{}' is not 2D, cannot keepunique it!".format(colname))
		


def groupreshape(cat, groupcolnames):
	"""
	Rearranges the data in the table so to have only one row per different value combination of the diffcolnames.
	For each of these rows, all the data is collected in the second dimension.
	Note that this seems to lead to a sorting of the table rows according to the groupcolnames!
	No problem with this, but be aware.
	
	
	Ideally these groupcolanmes would not contain floats, but integers. But it seems to work with simple floats as well.
	
	"""
	
	# Start
	logger.info("Determining groups for {}...".format(groupcolnames))
	
	# Check that groupcolnames are 1D columns
	for colname in groupcolnames:
		if cat[colname].ndim != 1:
			raise RuntimeError("Only 1D columns are allowed as groupcolnames! '{}' is not.".format(colname))
	
	# Use the table group functionality
	groupcat = cat.group_by(groupcolnames)
	
	ngroups = len(groupcat.groups.indices) -1
	logger.info("Reshaping data in {} groups...".format(ngroups))
	
	collapsedgroups = []
	for (i, group) in enumerate(groupcat.groups):
		
		if i%10 == 0:
			logger.info("Working on group {}/{} of length {}...".format(i+1, len(groupcat.groups), len(group)))
		collapsedgroup = collapse(group) # This is a Table
		assert len(collapsedgroup) == 1
		
		collapsedgroups.append(collapsedgroup)
	
	# Some assert checks
	# Columns should have exactly the same dimensions in all of the collapsedgroups:
	colshapes = [collapsedgroups[0][col].shape for col in collapsedgroups[0].colnames]
	for t in collapsedgroups:
		assert t.colnames == collapsedgroups[0].colnames
		assert [t[col].shape for col in t.colnames] == colshapes
	
	# Now we can simply vstack this stuff.
	logger.info("Done with reshaping, now stacking...")
	outcat = astropy.table.vstack(collapsedgroups, join_type=u'exact', metadata_conflicts=u'error')
	assert len(outcat) == ngroups
	
	# At least for the groupcolnames, we only want to keep single values:
	keepunique(outcat, groupcolnames)
	
	return outcat
		



def fastgroupreshape(cat, groupcolnames):
	"""
	Same as groupreshape, but way faster.
	Oh boy, should have implemented that earlier...
	I guess that it's not even less safe.
	
	"""
	# Start
	logger.info("Determining groups for {}...".format(groupcolnames))
	
	# Check that groupcolnames are 1D columns
	for colname in groupcolnames:
		if cat[colname].ndim != 1:
			raise RuntimeError("Only 1D columns are allowed as groupcolnames! '{}' is not.".format(colname))
	
	# Use the table group functionality
	groupcat = cat.group_by(groupcolnames)
	
	
	ngroups = len(groupcat.groups.indices) -1
	logger.info("Found {} groups".format(ngroups))
	
	groupsize = len(groupcat.groups[0])
	logger.info("Groups seem to have a length of {}".format(groupsize))
	
	if groupsize * ngroups != len(groupcat):
		raise RuntimeError("Sorry, weird structure!")

	logger.info("Aggregating the groups...")
	# OK using the groupcat.groups.aggregate function works, but is still slow.
	# aggcat = groupcat.groups.aggregate(lambda x: x.reshape(1,-1))
	# Instead,  we do the reshaping column by column.
	# groupcat is sorted according to the groups, we exploit this.
	# The keepunique at the end is a good check for errors.
	newcat = astropy.table.Table()
	for colname in groupcat.colnames:
		#print "reshaping {} ...".format(colname)
		newcat[colname] = groupcat[colname].reshape((ngroups, groupsize))

	logger.info("Keepuniquing the groupcolnames...")
	# At least for the groupcolnames, we only want to keep single values:
	keepunique(newcat, groupcolnames)

	return newcat
	
	




#####
# OK these addstats and addrmsd should be moved out of here, maybe in calc.
#####


def addstats(cat, col, wcol=None, outcolprefix=None):
	"""
	Adds columns containing some statistics of the values in each cell of col.
	So this is for 2D columns. Togheter this the function "group", this replaces the old-style "groupstats".
	
	:param col: name of the (2D) column containing the data for which stats should be computed
	:param wcol: name of the column containing some weights to be used, if desired.
	
	"""
	if outcolprefix == None:
		outcolprefix = col
	
	suffixes = ["_mean", "_med", "_std", "_n"]
	if wcol is not None:
		suffixes.append("_wmean")
	
	outcolnames = [outcolprefix + suffix for suffix in suffixes]
	for outcolname in outcolnames:
		if outcolname in cat.colnames:
			raise RuntimeError("Column {} already exists, refusing to overwrite.".format(outcolname))
	
	if cat[col].ndim != 2:
		raise RuntimeError("Column '{}' is not 2-dimensional, stats do not make sense!".format(col))
	if wcol is not None:
		if cat[col].shape != cat[wcol].shape:
			raise RuntimeError("Data in col and wcol should have the same shape!") 
		
		
	logger.info("Adding stats for column {} of shape {}...".format(col, cat[col].shape))
	
	cat[outcolprefix + "_mean"] = np.ma.mean(cat[col], axis=1)
	cat[outcolprefix + "_med"] = np.ma.median(cat[col], axis=1)
	cat[outcolprefix + "_std"] = np.ma.std(cat[col], axis=1)
	cat[outcolprefix + "_n"] = np.ma.count(cat[col], axis=1)
	if wcol is not None:
		# We add the weighted mean
		cat[outcolprefix + "_wmean"] = np.ma.mean(cat[col] * cat[wcol], axis=1) / np.ma.mean(cat[wcol], axis=1)
		# And the weight that can be attributed to this weighted mean (i.e., sum of weights)
		cat[outcolprefix + "_wmeanw"] = np.ma.sum(cat[wcol], axis=1)
	

def addrmsd(cat, colm, colt, outcolprefix=None):
	"""
	For each row, adds the RMSD of the many measured values in colm with respect to the true value in colt.
	"""
	
	if outcolprefix == None:
		outcolprefix = colm

	if cat[colm].ndim != 2:
		raise RuntimeError("Column '{}' must be 2-dimensional !".format(colm))
	if cat[colt].ndim != 1:
		raise RuntimeError("Column '{}' is not 1-dimensional !".format(colm))

	colmdata = copy.deepcopy(cat[colm])
	coltdata = copy.deepcopy(cat[colt]).reshape((cat[colt].size, 1))

	outcolnames = [outcolprefix + suffix for suffix in ["_rmsd", "_bias"]]
	for outcolname in outcolnames:
		if outcolname in cat.colnames:
			raise RuntimeError("Column {} already exists, refusing to overwrite.".format(outcolname))
	
	logger.info("Adding RMSD and bias for column {} of shape {} with respect to {}...".format(colm, cat[colm].shape, colt))
	
	cat[outcolprefix + "_rmsd"] = np.sqrt(np.mean((colmdata - coltdata)**2.0, axis=1))
	cat[outcolprefix + "_bias"] = np.mean(colmdata - coltdata, axis=1)
	
	


def make2d(cat, cols):
	"""
	Transform the columns cols into 2D-columns by reshaping them to shape (len(cat), 1)
	
	:param cols: a list of column names
	
	I'm surprised that this seems to be required for element-wise operations between 2D and 1D columns.
	
	works in place
	
	note the keepunique function above
	"""
	
	lencat = len(cat)
	for col in cols:
		
		if cat[col].ndim == 1:
			reshaped = cat[col].reshape((lencat, 1))
			cat.remove_column(col)
			cat[col] = reshaped
		
		elif cat[col].ndim == 2: # We silently ignore columsn that are already 2D.
			continue
		
		else:
			logger.critical("Could not turn '{}' indo a 2D-column!".format(col))
			continue
	
	


class Selector:
	"""
	Provides a simple way of getting "configurable" sub-selections of rows from a table.
	"""
	
	def __init__(self, name, criteria):
		"""
		
		:param name: a short string describing this selector (like "star", "low_snr", ...)
		:param criteria: a list of tuples describing the criteria. Each of these tuples starts 
			with a string giving its type, followed by some arguments.
	
			Illustration of the available criteria (all limits are inclusive):
		
			- ``("in", "tru_rad", 0.5, 0.6)`` : ``"tru_rad"`` is between 0.5 and 0.6 ("in" stands for *interval*) and *not* masked
			- ``("max", "snr", 10.0)`` : ``"snr"`` is below 10.0 and *not* masked
			- ``("min", "adamom_flux", 10.0)`` : ``"adamom_flux"`` is above 10.0 and *not* masked
			- ``("inlist", "subfield", (1, 2, 3))`` : ``subfield`` is among the elements in the tuple or list (1,2,3) and *not* masked.
			- ``("is", "Flag", 2)`` : ``"Flag"`` is exactly 2 and *not* masked
			- ``("nomask", "pre_g1")`` : ``"pre_g1"`` is not masked
			- ``("mask", "snr")`` : ``"snr"`` is masked
		
		
		"""
		self.name = name
		self.criteria = criteria
	
	def __str__(self):
		"""
		A string describing the selector
		"""
		return "'%s' %s" % (self.name, repr(self.criteria))
	
	
	def combine(self, *others):
		"""
		Returns a new selector obtained by merging the current one with one or more others.

		:param others: provide one or several other selectors as arguments.

		.. note:: This does **not** modify the current selector in place! It returns a new one!
		"""
	
		combiname = "&".join([self.name] + [other.name for other in others])
	
		combicriteria = self.criteria
		for other in others:
			combicriteria.extend(other.criteria)
	
		return Selector(combiname, combicriteria)
		
	
	def select(self, cat):
		"""
		Returns a copy of cat with those rows that satisfy all criteria.
		
		:param cat: an astropy table
		
		"""
		
		if len(self.criteria) is 0:
			logger.warning("Selector %s has no criteria!" % (self.name))
			return copy.deepcopy(cat)
		
		passmasks = []
		for crit in self.criteria:
		
			if cat[crit[1]].ndim != 1:
				logger.warning("Selecting with multidimensional column ('{}', shape={})... hopefully you know what you are doing.".format(crit[1], cat[crit[1]].shape))
			
			if crit[0] == "in":
				if len(crit) != 4: raise RuntimeError("Expected 4 elements in criterion %s" % (str(crit)))
				passmask = np.logical_and(cat[crit[1]] >= crit[2], cat[crit[1]] <= crit[3])
				if np.ma.is_masked(passmask):
					passmask = passmask.filled(fill_value=False)
				# Note about the "filled": if crit[2] or crit[3] englobe the values "underneath" the mask,
				# some masked crit[1] will result in a masked "passmask"!
				# But we implicitly want to reject masked values here, hence the filled.
							
			elif crit[0] == "max":
				if len(crit) != 3: raise RuntimeError("Expected 3 elements in criterion %s" % (str(crit)))
				passmask = (cat[crit[1]] <= crit[2])
				if np.ma.is_masked(passmask):
					passmask = passmask.filled(fill_value=False)
			
			elif crit[0] == "min":
				if len(crit) != 3: raise RuntimeError("Expected 3 elements in criterion %s" % (str(crit)))
				passmask = (cat[crit[1]] >= crit[2])
				if np.ma.is_masked(passmask):
					passmask = passmask.filled(fill_value=False)
			
			elif crit[0] == "inlist":
				if len(crit) != 3: raise RuntimeError("Expected 3 elements in criterion %s" % (str(crit)))
				passmask = np.in1d(np.asarray(cat[crit[1]]), crit[2]) # This ignores any mask
				if np.ma.is_masked(passmask): # As the mask is ignored by in1d, this is probably worthless and will never happen
					passmask = passmask.filled(fill_value=False)
				# So we need to deal with masked elements manually:
				if hasattr(cat[crit[1]], "mask"): # i.e., if this column is masked:
					passmask = np.logical_and(passmask, np.logical_not(cat[crit[1]].mask))
						
			elif crit[0] == "is":
				if len(crit) != 3: raise RuntimeError("Expected 3 elements in criterion %s" % (str(crit)))
				passmask = (cat[crit[1]] == crit[2])
				if np.ma.is_masked(passmask):
					passmask = passmask.filled(fill_value=False)
					
			elif (crit[0] == "nomask") or (crit[0] == "mask"):
				if len(crit) != 2: raise RuntimeError("Expected 2 elements in criterion %s" % (str(crit)))
				if hasattr(cat[crit[1]], "mask"): # i.e., if this column is masked:
					if crit[0] == "nomask":
						passmask = np.logical_not(cat[crit[1]].mask)
					else:
						passmask = cat[crit[1]].mask
				else:
					logger.warning("Criterion %s is facing an unmasked column!" % (str(crit)))
					passmask = np.ones(len(cat), dtype=bool)
			
			else:
				raise RuntimeError("Unknown criterion %s" % (crit))
					
			logger.debug("Criterion %s of '%s' selects %i/%i rows (%.2f %%)" %
				(crit, self.name, np.sum(passmask), len(cat), 100.0 * float(np.sum(passmask))/float(len(cat))))
			
			assert len(passmask) == len(cat)
			passmasks.append(passmask) # "True" means "pass" == "keep this"
		
		# Combining the passmasks:
		passmasks = np.logical_not(np.column_stack(passmasks)) # "True" means "reject"
		combimask = np.logical_not(np.sum(passmasks, axis=1).astype(bool)) # ... and "True" means "keep this" again.
		
		logger.info("Selector '%s' selects %i/%i rows (%.2f %%)" %
				(self.name, np.sum(combimask), len(cat), 100.0 * float(np.sum(combimask))/float(len(cat))))
		
		return cat[combimask]



# Deprecated. Use feature.cutmasked if you need that.
#
#def cutmasked(cat, colnames, keep_all_columns=True):
#	"""
#	Returns those rows of table for which all of the given colnames are unmasked.
#	To do this, it "combines" the masks from the different colnames.
#	This is in principle close to what **numpy.ma.compress_rows** does.
#	But here we also log a bit about the masked columns.
#	
#	:param cat: an astropy table
#	:param colnames: a list of column names whose data must be available in the output
#	:param keep_all_columns: if False, the returned table only contains the given colnames.
#	
#	:returns: an astropy table		
#	"""
#	# Some trivial tests:
#	for colname in colnames:
#		if colname not in cat.colnames:
#			raise RuntimeError("The column '%s' is not available among %s" % (colname, str(cat.colnames)))
#		
#		#assert cat[colname].ndim == 1 # This function has not been tested for anything else...
#		# No, this is now beginnign to get implemented...
#			
#	if len(colnames) != len(list(set(colnames))):
#		raise RuntimeError("Strange, some colnames appear multiple times in %s" % (str(colnames)))
#	
#	# First, get a list of 1D masks, one per column
#	
#	masklist = []
#	
#	print "Terrible hack, just uses mask of first rea if 2D !"
#	
#	for colname in colnames:
#		
#		mask = np.array(np.ma.getmaskarray(np.ma.array(cat[colname])), dtype=bool)
#		assert mask.ndim in [1, 2]
#		if mask.ndim == 2:
#			mask = mask[:,0] # Just get the first realization..
#			
#		assert mask.ndim == 1
#		assert mask.size == len(cat)
#		masklist.append(mask)
#	
#	# We now group all the masks for all these columns.
#	masks = np.column_stack(masklist)
#	
#	# This is a 2D boolean array. Now we log some details about how many points are masked in each column.
#	for (i, colname) in enumerate(colnames):
#		mask = masks[:,i]
#		assert len(mask) == len(cat)
#		nbad = np.sum(mask)
#		logger.info("Column %20s: %5i (%5.2f %%) of the %i entries are masked" \
#			% ("'"+colname+"'", nbad, 100.0 * float(nbad) / float(len(cat)), len(cat)))
#	
#	# Now we combine the masks:
#	combimask = np.logical_not(np.sum(masks, axis=1).astype(bool)) # So "True" means "keep this".
#	assert combimask.size == len(cat)
#	ngood = np.sum(combimask)
#	nbad = combimask.size - ngood
#	logger.info("Combination: %i out of %i (%.2f %%) rows have masked values and will be disregarded" \
#		% (nbad, combimask.size, 100.0 * float(nbad) / float(combimask.size)))
#	
#	# We disregard the masked rows:
#	nomaskcat = cat[combimask] # Only works as combimask is a numpy array ! It would do something else with a list !
#	# Wow, this is extremely SLOW !
#	# Note that everything so far also works for columns which are **not** masked. They will remain maskless.
#	
#	# More asserts:
#	assert len(nomaskcat) == ngood
#	for colname in colnames:
#		if hasattr(nomaskcat[colname], "mask"): # We make this test only if the column is masked
#			#assert np.all(np.logical_not(nomaskcat[colname].mask)) == True # Tests that all values are unmasked.
#			pass
#	
#	if not keep_all_columns:
#		nomaskcat.keep_columns(colnames)
#		# Moving this to before the slow "cat[combimask]" seems to not speedup this function -- strange ?
#	
#	return nomaskcat
#










import momentsml
import momentsml.plot

import os

from momentsml.tools.feature import Feature
import config

import matplotlib
import matplotlib.pyplot as plt

import numpy as np


cat = momentsml.tools.io.readpickle(os.path.join(config.simmeasdir, config.datasets["vo"], "groupmeascat.pkl"))


mask = np.array(cat["adamom_g1"].mask, dtype=bool) # This is 2D, 1 means "is masked"

print mask.shape
print np.sum(mask)

print float(np.sum(mask)) / float(mask.size)


exit()


nrea = mask.shape[1]

tru_flux = np.array([cat["tru_flux"] for i in range(nrea)]).transpose()
high_flux = tru_flux > 500.0

mask = np.logical_and(mask, high_flux)


fail_tru_g1s = cat["tru_g1"][mask]
fail_tru_g2s = cat["tru_g2"][mask]

#fail_tru_s1s = cat["tru_s1"][cat["adamom_g1"].mask]
#fail_tru_s2s = cat["tru_s2"][cat["adamom_g1"].mask]


fail_tru_ixs = cat["ix"][mask]

tru_iys = np.array([cat["iy"] for i in range(nrea)]).transpose()
fail_tru_iys = tru_iys[mask]

tru_flux = np.array([cat["tru_flux"] for i in range(nrea)]).transpose()
tru_rad = np.array([cat["tru_rad"] for i in range(nrea)]).transpose()
fail_tru_flux = tru_flux[mask]


fail_tru_rad = tru_rad[mask]


#print momentsml.tools.table.info(cat)

#exit()

"""
plt.scatter(fail_tru_ixs, fail_tru_g1s, c=fail_tru_flux)
plt.colorbar()
plt.show()
"""

plt.scatter(fail_tru_g1s, fail_tru_g2s, c=fail_tru_rad)
plt.title("adamom failures for bright sources (color=rad)")
plt.xlabel("g1")
plt.ylabel("g2")
plt.colorbar()
plt.show()






import momentsml
import momentsml.plot

import os

from momentsml.tools.feature import Feature
import config

import matplotlib
import matplotlib.pyplot as plt

import numpy as np


cat = momentsml.tools.io.readpickle(os.path.join(config.simmeasdir, config.datasets["tp"], "groupmeascat.pkl"))
#print momentsml.tools.table.info(cat)

fig = plt.figure(figsize=(8, 8))

ax = fig.add_subplot(2, 2, 1)
momentsml.plot.scatter.scatter(ax, cat, Feature("snr", rea=1),  Feature("adamom_failfrac"), sidehists=True)

ax = fig.add_subplot(2, 2, 2)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_sb"), Feature("tru_rad"), Feature("adamom_failfrac"))


plt.tight_layout()
plt.show()
plt.close(fig) # Helps releasing memory when calling in large loops.





import numpy as np
import tenbilac


import logging
logging.basicConfig(level=logging.INFO)


ni = 1

wnet = tenbilac.wnet.WNet(ni=ni, nhs=[3, 3], name="test", netokwargs={"onlyid":True})

#print wnet.nparams()
#print wnet.report()


ref = wnet.get_params_ref()

wnet.addnoise()

#print ref
#print wnet.report()
wnet.setini()
print wnet.report()



ncas = 3
nrea = 5
no = 1

inputs = np.random.randn(nrea * ni * ncas).reshape((nrea, ni, ncas))


#print inputs

outputs = wnet.run(inputs)


#print outputs

targets = np.ones(no * ncas).reshape((no, ncas))

# There is no by default weights, so let's initialise the weights to ones:
weights = np.ones_like(outputs)

print tenbilac.err.msbw(outputs, targets, auxinputs=weights)

"""
print "ouputs shape ", outputs.shape



targets = np.ones(no * ngal).reshape((no, ngal))


net.train(inputs, targets, tenbilac.err.msb, maxiter=3)



print net.report()
"""



"""
Validation plot on mixed-galaxies data, with optional weights
"""

import momentsml
import momentsmlgreat3
import momentsml.plot
from momentsml.tools.feature import Feature
import matplotlib.pyplot as plt
import numpy as np

import config

import os
import logging
logging.basicConfig(format=config.loggerformat, level=logging.INFO)
logger = logging.getLogger(__name__)



def main():

	print "This is no longer a standalone script"

	


def plot(cat, component, filepath=None, title=None):
	
	#rea = "all"
	rea = -20
	ebarmode = "scatter"
	
	# Adding weights if absent:
	if not "pre_s{}w".format(component) in cat.colnames:
		
		# First putting all weights to 1.0:
		cat["pre_s{}w".format(component)] = np.ones(cat["adamom_g1"].shape)
		
		"""
		# Keeping only the best half of SNR
		momentsml.tools.table.addstats(cat, "snr")
		for row in cat:
			row["pre_s{}w".format(component)] = np.array(row["snr"] > row["snr_med"], dtype=np.float)
		"""
		
		# Keeping the best half of sigma
		momentsml.tools.table.addstats(cat, "adamom_sigma")
		for row in cat:
			row["pre_s{}w".format(component)] = np.array(row["adamom_sigma"] > row["adamom_sigma_med"], dtype=np.float)
		
		
		
		#print momentsml.tools.table.info(cat)
		
		cat["pre_s{}w_norm".format(component)] = cat["pre_s{}w".format(component)] / np.max(cat["pre_s{}w".format(component)])
	
	momentsml.tools.table.addrmsd(cat, "pre_s{}".format(component), "tru_s{}".format(component))

	momentsml.tools.table.addstats(cat, "pre_s{}".format(component), "pre_s{}w".format(component))
	cat["pre_s{}_wbias".format(component)] = cat["pre_s{}_wmean".format(component)] - cat["tru_s{}".format(component)]
	
	pre_scw = Feature("pre_s{}w".format(component), rea=rea)
	pre_scw_norm = Feature("pre_s{}w_norm".format(component), rea=rea)
	
	pre_sc = Feature("pre_s{}".format(component), rea=rea)
	
	pre_sc_bias = Feature("pre_s{}_bias".format(component))
	pre_sc_wbias = Feature("pre_s{}_wbias".format(component))
	
	pre_sc_mean = Feature("pre_s{}_mean".format(component), -0.13, 0.13)
	pre_sc_wmean = Feature("pre_s{}_wmean".format(component), -0.13, 0.13)
	
	tru_sc = Feature("tru_s{}".format(component), -0.13, 0.13)
		


	fig = plt.figure(figsize=(20, 12))
	if title is not None:
		fig.suptitle(title)

	ax = fig.add_subplot(3, 4, 1)
	momentsml.plot.scatter.scatter(ax, cat, tru_sc, pre_sc, pre_scw)
	
	ax = fig.add_subplot(3, 4, 5)
	if component ==1:
		theother=Feature("tru_g2", rea=rea)
	else:
		theother=Feature("tru_g1", rea=rea)
	momentsml.plot.scatter.scatter(ax, cat, Feature("snr", rea=rea), pre_scw, theother)
	
	ax = fig.add_subplot(3, 4, 6)
	momentsml.plot.hist.hist(ax, cat, pre_scw)

	ax = fig.add_subplot(3, 4, 7)
	momentsml.plot.scatter.scatter(ax, cat, Feature("adamom_sigma", rea=rea), Feature("adamom_flux", rea=rea), featc=pre_scw)
	
	ax = fig.add_subplot(3, 4, 8)
	momentsml.plot.scatter.scatter(ax, cat, Feature("tru_rad", rea=rea), Feature("tru_flux", rea=rea), featc=pre_scw)



	ax = fig.add_subplot(3, 4, 9)
	momentsml.plot.bin.bin(ax, cat, tru_sc, pre_sc_bias, showidline=True,  metrics=True, yisres=True)
	ax.set_title("Without weights")

	ax = fig.add_subplot(3, 4, 10)
	momentsml.plot.scatter.scatter(ax, cat, tru_sc, pre_sc_bias, showidline=True, metrics=True, yisres=True)
	ax.set_title("Without weights")
	
	ax = fig.add_subplot(3, 4, 11)
	momentsml.plot.bin.bin(ax, cat, tru_sc, pre_sc_wbias, showidline=True, metrics=True, yisres=True)
	ax.set_title("With weights")

	ax = fig.add_subplot(3, 4, 12)
	momentsml.plot.scatter.scatter(ax, cat, tru_sc, pre_sc_wbias, showidline=True, metrics=True, yisres=True)
	ax.set_title("With weights")

	
	plt.tight_layout()

	if filepath:
		logger.info("Writing plot to '{}'...".format(filepath))
		plt.savefig(filepath)
	else:
		plt.show()
	plt.close(fig) # Helps releasing memory when calling in large loops.


if __name__ == "__main__":
    main()
	



import momentsml
import os
import config
import numpy as np

from momentsml.tools.feature import Feature

import matplotlib
import matplotlib.pyplot as plt


#####

catgems = momentsml.tools.io.readpickle(os.path.join(config.simmeasdir, "si-1-gems", "groupmeascat.pkl"))
catuni = momentsml.tools.io.readpickle(os.path.join(config.simmeasdir, "si-1-uni", "groupmeascat.pkl"))



#####


fig = plt.figure(figsize=(12, 12))


snr = Feature("snr", 0, 50, nicename="Measured S/N")
tru_rad = Feature("tru_rad", nicename="True half-light radius [pixel]")
tru_mag = Feature("tru_mag", 20, 25, nicename="Magnitude")
tru_sersicn = Feature("tru_sersicn", 0, 7, nicename="True Sersic index")

skystd = Feature("skystd")
skymed = Feature("skymed")
adamom_flux = Feature("adamom_flux")
adamom_logflux = Feature("adamom_logflux")
adamom_sigma = Feature("adamom_sigma")
adamom_g1 = Feature("adamom_g1")
adamom_g2 = Feature("adamom_g2")
adamom_g = Feature("adamom_g")
adamom_rho4 = Feature("adamom_rho4")



#snr = Feature("snr", nicename="Measured S/N")
#tru_rad = Feature("tru_rad", nicename="True half-light radius [pixel]")
#tru_mag = Feature("tru_mag", nicename="Magnitude")
#tru_sersicn = Feature("tru_sersicn", nicename="True Sersic index")


ax = fig.add_subplot(3, 3, 1)
momentsml.plot.contour.simobs(ax, catgems, catuni, snr, adamom_logflux)

ax = fig.add_subplot(3, 3, 2)
momentsml.plot.contour.simobs(ax, catgems, catuni, skystd, skymed)

ax = fig.add_subplot(3, 3, 3)
momentsml.plot.contour.simobs(ax, catgems, catuni, adamom_logflux, adamom_sigma)

ax = fig.add_subplot(3, 3, 4)
momentsml.plot.contour.simobs(ax, catgems, catuni, adamom_g, adamom_rho4)

ax = fig.add_subplot(3, 3, 5)
momentsml.plot.contour.simobs(ax, catgems, catuni, adamom_g1, adamom_g2)


ax = fig.add_subplot(3, 3, 7)
momentsml.plot.hist.hist(ax, catgems, tru_rad)
momentsml.plot.hist.hist(ax, catuni, tru_rad, color="green")

ax = fig.add_subplot(3, 3, 8)
momentsml.plot.hist.hist(ax, catgems, adamom_sigma)
momentsml.plot.hist.hist(ax, catuni, adamom_sigma, color="green")


plt.tight_layout()
plt.show()
plt.close(fig) # Helps releasing memory when calling in large loops.






import numpy as np
import tenbilac
import matplotlib.pyplot as plt

import logging
logging.basicConfig(level=logging.INFO)


from matplotlib import rc
rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
## for Palatino and other serif fonts use:
#rc('font',**{'family':'serif','serif':['Palatino']})
rc('text', usetex=True)



d = tenbilac.utils.readpickle("data.pkl")

params_normer = d["params_normer"]
obs_normer = d["obs_normer"]
test_obs = d["test_obs"]
val_params = d["val_params"]


# One one plot, we show the actual training data points.
# We modify those arrays to get everything into flat 1D arrays
mod_train_params = np.concatenate(d["train_params"].shape[0]*[d["train_params"].T]).T[0]
mod_train_obs = d["train_obs"].ravel()

# Showing all of them looks ugly, just a few:
showtrainindices = np.arange(mod_train_params.size)
np.random.shuffle(showtrainindices)
showtrainindices = showtrainindices[:10000]
mod_train_params = mod_train_params[showtrainindices]
mod_train_obs = mod_train_obs[showtrainindices]


train_msb = tenbilac.utils.readpickle("train_msb.pkl")
test_preds_msb = params_normer.denorm(train_msb.net.run(d["normed_test_obs"]))
val_preds_msb = params_normer.denorm(train_msb.net.run(d["normed_val_obs"]))
#firstreapreds_msb = params_normer.denorm(train_msb.net.run(normuniobs[0]))
val_biases_msb = np.mean(val_preds_msb, axis=0) - d["val_params"]


train_mse = tenbilac.utils.readpickle("train_mse.pkl")
test_preds_mse = params_normer.denorm(train_mse.net.run(d["normed_test_obs"]))
val_preds_mse = params_normer.denorm(train_mse.net.run(d["normed_val_obs"]))
#firstreapreds_mse = params_normer.denorm(train_mse.net.run(normuniobs[0]))
val_biases_mse = np.mean(val_preds_mse, axis=0) - d["val_params"]


train_avg = tenbilac.utils.readpickle("train_avg.pkl")
test_preds_avg = params_normer.denorm(train_avg.net.run(d["normed_test_obs"]))
val_preds_avg = params_normer.denorm(train_avg.net.run(d["normed_val_obs"]))
#firstreapreds_avg = params_normer.denorm(train_avg.net.run(normuniobs[0]))
val_biases_avg = np.mean(val_preds_avg, axis=0) - d["val_params"]



trutheta = np.linspace(0.0, 2.2, 100)
trud = np.sqrt(1.0 + np.square(trutheta))

fig = plt.figure(figsize=(9, 4.5))

#color_cycle = ['#377eb8', '#ff7f00', '#4daf4a',
#                  '#f781bf', '#a65628', '#984ea3',
#                  '#999999', '#e41a1c', '#dede00']

color_cycle = ["#1b9e77", "#d95f02", "#7570b3"]
# From colorbrewer, safe for colorblind

ax = fig.add_subplot(1, 2, 1)
ax.plot(mod_train_params, mod_train_obs, marker=".", color="gray", ls="None", ms=2, label="Training data samples")
ax.plot(test_preds_mse.T, test_obs.T, ls="-", color=color_cycle[0], label="Trained with MSE", lw=1.5)
ax.plot(test_preds_avg.T, test_obs.T, ls=":", color=color_cycle[1], label=r"Trained on $\langle d \rangle$", lw=1.5)
ax.plot(test_preds_msb.T, test_obs.T, ls="-.", color=color_cycle[2], label="Trained with MSB", lw=1.5)
ax.plot(trutheta, trud, ls="-", color="black", dashes=(5, 5), lw=2.0, label=r"$d = \sqrt{1 + \theta^2}$")
ax.set_xlabel(r"$\theta$ $\mathrm{and}$ $\hat{\theta}$", fontsize=18)
ax.set_ylabel(r"$d$", fontsize=18)
ax.set_xlim(-1.2, 2.4)
ax.set_ylim(0.5, 3.0)
ax.legend(loc=2, fontsize=12, markerscale=4, numpoints=1)


ax = fig.add_subplot(1, 2, 2)
ax.plot(val_params.T, val_biases_mse.T, marker=".", color=color_cycle[0], label="Trained with MSE", ls="None", ms=2)
ax.plot(val_params.T, val_biases_avg.T, marker=".", color=color_cycle[1], label=r"Trained on $\langle d \rangle$", ls="None", ms=2)
ax.plot(val_params.T, val_biases_msb.T, marker=".", color=color_cycle[2], label="Trained with MSB", ls="None", ms=2)
#ax.text(-0.1, val_biases_mse[0,0], "MSE", color=color_cycle[0])
#ax.text(-0.1, val_biases_avg[0,0], r"$\langle d \rangle$", color=color_cycle[1])
#ax.text(-0.1, val_biases_msb[0,0], "MSB", color=color_cycle[2])


ax.axhline(0.0, color="black", lw=2)
ax.set_xlabel(r"$\theta$", fontsize=18)
ax.set_ylabel(r"$\langle \hat{\theta} - \theta \rangle$", fontsize=18)
ax.set_xlim(0.15, 2.1)
ax.set_ylim(-0.3, 0.3)
ax.legend(fontsize=12, markerscale=4, numpoints=1)



plt.tight_layout()
#plt.show()
plt.savefig("MSB_demo.pdf")





import numpy as np
import tenbilac


import logging
logging.basicConfig(level=logging.INFO)


ni = 4
ngal = 1
nrea = 2
no = 4


net = tenbilac.net.Net(ni=ni, nhs=[4, -6, 4], no=no, onlyid=True)

net.setidentity(onlyn=2)

print net.report()


inputs = np.random.randn(nrea * ni * ngal).reshape((nrea, ni, ngal))

# Setting the first rea input to some well-known stuff:
inputs[0,0,:] = 0.8
inputs[0,1,:] = 0.3
inputs[0,2,:] = 0.1
inputs[0,3,:] = -0.3


#inputs *= 50.0 
#inputs += 1000.0


print "inputs shape", inputs.shape
print inputs

outputs = net.run(inputs)

print "outputs shape", outputs.shape
print outputs



import matplotlib
matplotlib.use("AGG")

import os

import momentsml.learn

import config

import logging
logger = logging.getLogger(__name__)

import argparse

parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
parser.add_argument("snrcut", type=int, help="<S/N>-cut to apply to the training cases")
parser.add_argument("target", type=str, help="Target shear (s) or ellipticity (e)")
args = parser.parse_args()

logger.info("Recieved arguments snrcut:{} and target:{}".format(args.snrcut, args.target))
	

# Getting the path to the correct directories
measdir = os.path.join(config.simmeasdir, config.datasets["tp"])
traindir = os.path.join(config.traindir, config.datasets["tp"])
	
# And to the catalogue
traincatpath = os.path.join(measdir, "groupmeascat.pkl")
cat = momentsml.tools.io.readpickle(traincatpath)
#print momentsml.tools.table.info(cat)


if args.target is "e":
	cat["tru_s1"] = cat["tru_g1"]
	cat["tru_s2"] = cat["tru_g2"]
elif args.target is "s":
	pass
else:
	raise ValueError("Unknown type")


select=True
if select:
	logger.warning("Selection of cases is activated!")
	momentsml.tools.table.addstats(cat, "snr")
	s = momentsml.tools.table.Selector("snrcut", [
		#("max", "adamom_failfrac", 0.01),
		("min", "snr_mean", args.snrcut),
	])
	cat = s.select(cat)



# Running the training
dirnames = momentsml.learn.tenbilacrun.train(cat, config.shearconflist, traindir)






import os
import momentsml
import config
import matplotlib.pyplot as plt
from momentsml.tools.feature import Feature



import astropy.table
import numpy as np
import galsim




gemspath = os.path.join(config.gemsdir, "gems_20090807.fits")
catpath = os.path.join(config.workdir, "cat.pkl")


cat = astropy.table.Table.read(gemspath)
#print cat.colnames

print len(cat)

cat.keep_columns(['ST_FLUX_RADIUS', 'ST_N_GALFIT', 'ST_MAG_GALFIT', 'ST_RE_GALFIT', 'ST_Q_GALFIT', "GEMS_FLAG", 'ST_MAG_BEST'])

cat["magdiff"] = np.fabs(cat["ST_MAG_BEST"] - cat["ST_MAG_GALFIT"])

# We select galaxies
s = momentsml.tools.table.Selector("select", [
		("is", "GEMS_FLAG", 4),
		("max", "magdiff", 0.5),
		("in", "ST_MAG_GALFIT", 20.5, 25.0),
		("in", "ST_N_GALFIT", 0.3, 6.0),
		("in", "ST_RE_GALFIT", 0.0, 12.0 / (0.03 / 0.1)), # Between 0 and 12 VIS pixels
	])

cat = s.select(cat)


cat["tru_rad"] = (0.03 / 0.1) * cat["ST_RE_GALFIT"] # GEMS is 0.03 arcsec per pixel, and we want VIS pixels of 0.1 arcsec
cat["tru_mag"] = cat["ST_MAG_GALFIT"]
cat["tru_sersicn"] = cat["ST_N_GALFIT"]

# We reject any nans in the field that we want
s = momentsml.tools.table.Selector("nonan", [
		("nomask", "tru_rad"),
		("nomask", "tru_mag"),
		("nomask", "tru_sersicn"),
	])
cat = s.select(cat)


fig = plt.figure(figsize=(20, 10))


maggal = Feature("ST_MAG_GALFIT")
magsex = Feature("ST_MAG_BEST")

sersicn = Feature("ST_N_GALFIT")
radsex = Feature("ST_FLUX_RADIUS")
radgal = Feature("ST_RE_GALFIT")


tru_mag = Feature("tru_mag", nicename="VIS magnitude")
tru_rad = Feature("tru_rad", nicename="Half-light radius [VIS pix]")
tru_sersicn = Feature("tru_sersicn", nicename="Sersic index")



ax = fig.add_subplot(2, 4, 1)
momentsml.plot.scatter.scatter(ax, cat, magsex,  maggal, radsex, sidehists=True)

ax = fig.add_subplot(2, 4, 2)
momentsml.plot.scatter.scatter(ax, cat, radsex,  radgal, maggal)

ax = fig.add_subplot(2, 4, 3)
momentsml.plot.scatter.scatter(ax, cat, maggal,  radgal, sidehists=True)

ax = fig.add_subplot(2, 4, 4)
magbinlims = np.arange(np.min(cat["tru_mag"]), np.max(cat["tru_mag"])+0.1, 0.5)
magbincents = 0.5*(magbinlims[:-1] + magbinlims[1:])
magcounts, bla = np.histogram(cat["tru_mag"], bins=magbinlims)
ax.plot(magbincents, magcounts, label="GEMS")
ax.set_yscale("log")
for magbinlim in magbinlims:
	ax.axvline(magbinlim, color="gray", lw=0.5)
ax.legend()



ax = fig.add_subplot(2, 4, 5)
momentsml.plot.hist.hist(ax, cat, tru_sersicn)

ax = fig.add_subplot(2, 4, 6)
momentsml.plot.scatter.scatter(ax, cat, tru_mag, tru_rad, tru_sersicn)

ax = fig.add_subplot(2, 4, 7)
momentsml.plot.scatter.scatter(ax, cat, tru_mag, tru_rad, sidehists=True)


plt.tight_layout()
plt.show()
plt.close(fig) # Helps releasing memory when calling in large loops.

momentsml.tools.io.writepickle(cat, catpath)






"""
This measfct adds a SNR to the catalog, computed from existing columns.
It does not measure the images.

"""

import astropy.table
import numpy as np
import copy

import logging
logger = logging.getLogger(__name__)

def measfct(catalog, prefix="", fluxcol="adamom_flux", sigmacol="adamom_sigma", stdcol="skymad", sizecol=None, gain=None, gaincol=None, aper=3.0):
	"""
	We assume here that the images are in ADU, and that 
	gain is given in electron / ADU.
	
	:param fluxcol:
	:param sigmacol:
	:param stdcol:
	:param gain:
	:param gaincol:
	
	:param aper: the radius of the effective area is hlr * aper. So aper = 3, the default value, means an aperture with a diameter of 3 times the half-light-diameter.
		This gives values close to FLUX_AUTO / FLUXERR_AUTO
	
	
	"""
	
	if gain != None and gaincol != None:
		raise RuntimeError("Please provide either gain or gaincol, not both!")
		
	output = astropy.table.Table(copy.deepcopy(catalog), masked=True)
	
	if gain != None:
		gain = np.abs(gain)
		logger.info("Now computing SNR assuming a gain of {0:.2} electrons per ADU.".format(gain))	
		sourcefluxes = output[fluxcol] * gain
		skynoisefluxes = (output[stdcol] * gain) ** 2 # per pixel
	
	elif gaincol != None:
		logger.info("Now computing SNR using the gain from column '{}'".format(gaincol))	
		gain = np.abs(output[gaincol])
		sourcefluxes = output[fluxcol] * gain
		skynoisefluxes = (output[stdcol] * gain) ** 2 # per pixel
	
	else:
		raise RuntimeError("Please provide a gain or gaincol!")
		
	
	if sizecol is None:
		areas = np.pi * (output[sigmacol] * aper * 1.1774) ** 2 # 1.1774 x sigma = r half light
	else: 
		areas = output[sizecol]
	
	noises = np.sqrt(sourcefluxes + areas * skynoisefluxes)
	
	snrcol = prefix + "snr"
	
	output[snrcol] = sourcefluxes / noises
	
	return output
	
	
	
	
	



import matplotlib
#matplotlib.use("AGG")
matplotlib.use("pdf")

import momentsml
import momentsml.tools
import momentsml.plot
from momentsml.tools.feature import Feature

import momentsmlgreat3
import astropy


import config
import numpy as np
import os
import matplotlib.pyplot as plt
import matplotlib.ticker as plticker
from matplotlib.ticker import AutoMinorLocator, LogLocator

from matplotlib import rc
rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
## for Palatino and other serif fonts use:
#rc('font',**{'family':'serif','serif':['Palatino']})
rc('text', usetex=True)

import logging
logging.basicConfig(format=config.loggerformat, level=logging.DEBUG)
logger = logging.getLogger(__name__)

from matplotlib.ticker import Locator


#Fixing a minor locator symlog bug: 
#http://stackoverflow.com/questions/20470892/how-to-place-minor-ticks-on-symlog-scale
class MinorSymLogLocator(Locator):
	"""
	Dynamically find minor tick positions based on the positions of
	major ticks for a symlog scaling.
	"""
	def __init__(self, linthresh):
		"""
		Ticks will be placed between the major ticks.
		The placement is linear for x between -linthresh and linthresh,
		otherwise its logarithmically
		"""
		self.linthresh = linthresh

	def __call__(self):
		'Return the locations of the ticks'
		majorlocs = self.axis.get_majorticklocs()

		# iterate through minor locs
		minorlocs = []

		# handle the lowest part
		for i in xrange(1, len(majorlocs)):
			majorstep = majorlocs[i] - majorlocs[i-1]
			if abs(majorlocs[i-1] + majorstep/2) < self.linthresh:
				ndivs = 10
			else:
				ndivs = 9
			minorstep = majorstep / ndivs
			locs = np.arange(majorlocs[i-1], majorlocs[i], minorstep)[1:]
			minorlocs.extend(locs)

		return self.raise_if_exceeds(np.array(minorlocs))

	def tick_values(self, vmin, vmax):
		raise NotImplementedError('Cannot get tick locations for a '
								  '%s type.' % type(self))



catpath = config.great3.path("summary_{}.pkl".format(config.predcode))
#catpath = config.great3.path("{}_summary_{}.pkl".format(config.datasets["mimic-great3"], config.predcode))


cat = momentsml.tools.io.readpickle(catpath)


# Computing residuals

cat["pre_s1_res"] = cat["pre_s1"] - cat["tru_s1"]
cat["pre_s2_res"] = cat["pre_s2"] - cat["tru_s2"]

cat["pre_s1w_res"] = cat["pre_s1w"] - cat["tru_s1"]
cat["pre_s2w_res"] = cat["pre_s2w"] - cat["tru_s2"]



# Computing PSF rotated errors
# angles:
rotations = .5 * np.arctan2(cat["tru_psf_g2"], cat["tru_psf_g1"]) # Note the inverted order of components
		

# rotating  predictions:
cat["pre_s+"] = cat["pre_s1"] * np.cos(-2. * rotations) - cat["pre_s2"] * np.sin(-2. * rotations)
cat["pre_sx"] = cat["pre_s1"] * np.sin(-2. * rotations) + cat["pre_s2"] * np.cos(-2. * rotations)

# rotating weighted predictions:
cat["pre_s+w"] = cat["pre_s1w"] * np.cos(-2. * rotations) - cat["pre_s2w"] * np.sin(-2. * rotations)
cat["pre_sxw"] = cat["pre_s1w"] * np.sin(-2. * rotations) + cat["pre_s2w"] * np.cos(-2. * rotations)

# rotating truth:
cat["tru_s+"] = cat["tru_s1"] * np.cos(-2. * rotations) - cat["tru_s2"] * np.sin(-2. * rotations)
cat["tru_sx"] = cat["tru_s1"] * np.sin(-2. * rotations) + cat["tru_s2"] * np.cos(-2. * rotations)

# And again the residuals
cat["pre_s+_res"] = cat["pre_s+"] - cat["tru_s+"]
cat["pre_sx_res"] = cat["pre_sx"] - cat["tru_sx"]

cat["pre_s+w_res"] = cat["pre_s+w"] - cat["tru_s+"]
cat["pre_sxw_res"] = cat["pre_sxw"] - cat["tru_sx"]


#print momentsml.tools.table.info(cat)
#mets = momentsmlgreat3.utils.metrics(cat, ("tru_s1", "tru_s2"), ("pre_s1w", "pre_s2w"), psfgcols=("tru_psf_g1", "tru_psf_g2"))


def labeloutliers(ax, cat, pre, tru, thr=0.01, whichones=None):
	"""
	whichones supercedes threshold
	"""
	cat["offset"] = cat[pre] - cat[tru]
	for row in cat:
		
		if whichones is not None:
			if row["subfield"] not in whichones:
				continue
		elif np.fabs(row["offset"]) < thr:
			continue
		
		# We are here only if we want to draw that label
		#print row[tru], row[pre], str(row["subfield"])
		ax.text(row[tru]+0.003, row[pre] - row[tru], str(row["subfield"]), fontsize=8)
	

def addmetrics(ax, xfeat, yfeat):
	metrics = momentsml.tools.metrics.metrics(cat, xfeat, yfeat, pre_is_res=True)
	line1 = r"$\mathrm{RMSD} = %.5f $" % (metrics["rmsd"])
	#line2 = r"$m: %.1f +/- %.1f; c: %.1f +/- %.1f$" % (metrics["m"]*1000.0, metrics["merr"]*1000.0, metrics["c"]*1000.0, metrics["cerr"]*1000.0)
	#line2 = r"$m=%.1f \pm %.1f; c=%.1f \pm %.1f \, [10^{-3}]$" % (metrics["m"]*1000.0, metrics["merr"]*1000.0, metrics["c"]*1000.0, metrics["cerr"]*1000.0)
	line2 = r"$10^3 \mu=%.1f \pm %.1f $" % (metrics["m"]*1000.0, metrics["merr"]*1000.0)
	line3 = r"$10^3 c=%.1f \pm %.1f $" % (metrics["c"]*1000.0, metrics["cerr"]*1000.0)
	
	ax.annotate(line1, xy=(0.0, 1.0), xycoords='axes fraction', xytext=(8, -7), textcoords='offset points', ha='left', va='top', fontsize=8)
	ax.annotate(line2, xy=(0.0, 1.0), xycoords='axes fraction', xytext=(8, -16), textcoords='offset points', ha='left', va='top', fontsize=8)
	ax.annotate(line3, xy=(0.0, 1.0), xycoords='axes fraction', xytext=(8, -25), textcoords='offset points', ha='left', va='top', fontsize=8)

	


sr = 0.06 # shear range
srp = 0.25 # shear range predictions
swrp = 0.02
symthres = 0.01
#subs = [1.1,2.,3.,4.,5.,6.,7.,8.,9.,-1.1,-2.,-3.,-4.,-5.,-6.,-7.,-8.,-9.]
#subs = [2.,3.,4.,5.,6.,7.,8.,9.,-2.,-3.,-4.,-5.,-6.,-7.,-8.,-9.]

fig = plt.figure(figsize=(9, 6))

fig.text(.005, .94, config.great3.get_branchacronym().upper(), fontdict={"fontsize":22})


paneltoplevel = 0.55
panelbottomlevel = 0.1
panelxw = 0.23
panelyw = 0.35

#cmap="coolwarm"
#cmap="viridis_r"
cmap="plasma_r"

# Little numbers
acro = config.great3.get_branchacronym()
if acro == "cgc":
	whichones = [2]
elif acro == "csc":
	#whichones = None
	whichones = []


#minorlocator = LogLocator(10,subs=subs[:])
minorlocator = MinorSymLogLocator(1e-1)

defaultmetrics = False

xaxislabelpad = -1

featc = Feature("psf_adamom_sigma", nicename=r"PSF $\mathtt{adamom\_sigma}$ [pix]")


ax = plt.axes([.09, paneltoplevel, panelxw, panelyw])
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s1", -sr, sr, nicename=r"True $g_1$"), Feature("pre_s1_res", -srp, srp, nicename=r"Predicted $g_1$ $-$ True $g_1$"), featc, yisres=True, cmap=cmap, metrics=defaultmetrics, showidline=True, hidecbar=True, idlinekwargs={"color":"black"})
labeloutliers(ax, cat, "pre_s1", "tru_s1", whichones=whichones)
ax.set_title("Without weights")
ax.title.set_position([.5, 1.1])
ax.set_yscale('symlog', linthreshy=symthres)
ax.xaxis.set_major_locator(plticker.MultipleLocator(base=0.05))
#ax.yaxis.set_minor_locator(plticker.MultipleLocator(base=0.002))
ax.yaxis.set_minor_locator(minorlocator)
ax.fill_between([-1, 1], -symthres, symthres, alpha=0.2, facecolor='darkgrey')
#cb = fig.colorbar(ax) 
#cb = ax.images[-1].colorbar
#cb.remove()
ax.yaxis.labelpad = xaxislabelpad
addmetrics(ax, Feature("tru_s1"), Feature("pre_s1_res"))


ax = plt.axes([.33, paneltoplevel, panelxw, panelyw])
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s1", -sr, sr, nicename=r"True $g_1$"), Feature("pre_s1w_res", -srp, srp, nicename=r"Predicted $g_1$ $-$ True $g_1$"), featc, yisres=True, cmap=cmap, metrics=defaultmetrics, showidline=True, hidecbar=True, idlinekwargs={"color":"black"})
labeloutliers(ax, cat, "pre_s1w", "tru_s1", thr=0.004, whichones=whichones)
ax.set_title("With weights")
ax.title.set_position([.5, 1.1])
ax.set_yscale('symlog', linthreshy=symthres)
ax.xaxis.set_major_locator(plticker.MultipleLocator(base=0.05))
#ax.yaxis.set_minor_locator(plticker.MultipleLocator(base=0.002))
ax.yaxis.set_minor_locator(minorlocator)
ax.fill_between([-1, 1], -symthres, symthres, alpha=0.2, facecolor='darkgrey')
#ax.yaxis.set_visible(False)
ax.set_yticklabels([])
ax.yaxis.label.set_visible(False)
addmetrics(ax, Feature("tru_s1"), Feature("pre_s1w_res"))


ax = plt.axes([.68, paneltoplevel, panelxw+0.02, panelyw])
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s+", -sr, sr, nicename=r"True $g_+$"), Feature("pre_s+w_res", -srp, srp, nicename=r"Predicted $g_+$ $-$ True $g_+$"), featc, yisres=True, cmap=cmap, metrics=defaultmetrics, showidline=True, idlinekwargs={"color":"black"})
labeloutliers(ax, cat, "pre_s+w", "tru_s+", thr=0.004, whichones=whichones)
ax.set_title("With weights")
ax.title.set_position([.5, 1.1])
ax.set_yscale('symlog', linthreshy=symthres)
ax.xaxis.set_major_locator(plticker.MultipleLocator(base=0.05))
#ax.yaxis.set_minor_locator(plticker.MultipleLocator(base=0.002))
ax.yaxis.set_minor_locator(minorlocator)
ax.fill_between([-1, 1], -symthres, symthres, alpha=0.2, facecolor='darkgrey')
#ax.yaxis.set_visible(False)
#ax.set_yticklabels([])
#ax.yaxis.label.set_visible(False)
ax.yaxis.labelpad = xaxislabelpad
addmetrics(ax, Feature("tru_s+"), Feature("pre_s+w_res"))

	
### Bottom row

ax = plt.axes([.09, panelbottomlevel, panelxw, panelyw])

momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s2", -sr, sr, nicename=r"True $g_2$"), Feature("pre_s2_res", -srp, srp, nicename=r"Predicted $g_2$ $-$ True $g_2$"), featc, yisres=True, cmap=cmap, metrics=defaultmetrics, showidline=True, hidecbar=True, idlinekwargs={"color":"black"})
labeloutliers(ax, cat, "pre_s2", "tru_s2", whichones=whichones)

ax.set_yscale('symlog', linthreshy=symthres)
ax.xaxis.set_major_locator(plticker.MultipleLocator(base=0.05))
#ax.yaxis.set_minor_locator(plticker.MultipleLocator(base=0.002))
ax.yaxis.set_minor_locator(minorlocator)
ax.fill_between([-1, 1], -symthres, symthres, alpha=0.2, facecolor='darkgrey')
#cb = fig.colorbar(ax) 
#cb = ax.images[-1].colorbar
#cb.remove()
ax.yaxis.labelpad = xaxislabelpad
addmetrics(ax, Feature("tru_s2"), Feature("pre_s2_res"))


ax = plt.axes([.33, panelbottomlevel, panelxw, panelyw])

momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s2", -sr, sr, nicename=r"True $g_2$"), Feature("pre_s2w_res", -srp, srp, nicename=r"Predicted $g_2$ $-$ True $g_2$"), featc, yisres=True, cmap=cmap, metrics=defaultmetrics, showidline=True, hidecbar=True, idlinekwargs={"color":"black"})
labeloutliers(ax, cat, "pre_s2w", "tru_s2", thr=0.004, whichones=whichones)

ax.set_yscale('symlog', linthreshy=symthres)
ax.xaxis.set_major_locator(plticker.MultipleLocator(base=0.05))
#ax.yaxis.set_minor_locator(plticker.MultipleLocator(base=0.002))
ax.yaxis.set_minor_locator(minorlocator)
ax.fill_between([-1, 1], -symthres, symthres, alpha=0.2, facecolor='darkgrey')
#ax.yaxis.set_visible(False)
ax.set_yticklabels([])
ax.yaxis.label.set_visible(False)
addmetrics(ax, Feature("tru_s2"), Feature("pre_s2w_res"))


ax = plt.axes([.68, panelbottomlevel, panelxw, panelyw])

momentsml.plot.scatter.scatter(ax, cat, Feature("tru_sx", -sr, sr, nicename=r"True $g_{\times}$"), Feature("pre_sxw_res", -srp, srp, nicename=r"Predicted $g_{\times}$ $-$ True $g_{\times}$"), featc, yisres=True, cmap=cmap, metrics=defaultmetrics, showidline=True, hidecbar=True, idlinekwargs={"color":"black"})
labeloutliers(ax, cat, "pre_sxw", "tru_sx", thr=0.004, whichones=whichones)
#labeloutliers(ax, cat, "pre_sx", "tru_sx")

ax.set_yscale('symlog', linthreshy=symthres)
ax.xaxis.set_major_locator(plticker.MultipleLocator(base=0.05))
#ax.yaxis.set_minor_locator(plticker.MultipleLocator(base=0.002))
ax.yaxis.set_minor_locator(minorlocator)
ax.fill_between([-1, 1], -symthres, symthres, alpha=0.2, facecolor='darkgrey')
#ax.yaxis.set_visible(False)
#ax.set_yticklabels([])
#ax.yaxis.label.set_visible(False)
ax.yaxis.labelpad = xaxislabelpad
addmetrics(ax, Feature("tru_sx"), Feature("pre_sxw_res"))





plotpath = config.great3.path("fig_2_{}.pdf".format(config.great3.get_branchacronym()))
plt.savefig(plotpath)
#plt.show()
print plotpath



#import matplotlib
#matplotlib.use("AGG")

import momentsml
import momentsml.tools
import momentsml.plot
from momentsml.tools.feature import Feature

import momentsmlgreat3
import astropy


import config
import numpy as np
import os
import matplotlib.pyplot as plt


import logging
logging.basicConfig(format=config.loggerformat, level=logging.DEBUG)
logger = logging.getLogger(__name__)

catpath = config.great3.path("summary_{}.pkl".format(config.predcode))
cat = momentsml.tools.io.readpickle(catpath)

cat["pre_s1_res"] = cat["pre_s1"] - cat["tru_s1"]
cat["pre_s2_res"] = cat["pre_s2"] - cat["tru_s2"]

cat["pre_s1w_res"] = cat["pre_s1w"] - cat["tru_s1"]
cat["pre_s2w_res"] = cat["pre_s2w"] - cat["tru_s2"]


#print momentsml.tools.table.info(cat)

mets = momentsmlgreat3.utils.metrics(cat, ("tru_s1", "tru_s2"), ("pre_s1w", "pre_s2w"), psfgcols=("tru_psf_g1", "tru_psf_g2"))


def labeloutliers(ax, cat, pre, tru, thr=0.01):
	cat["offset"] = cat[pre] - cat[tru]
	for row in cat:
		if np.fabs(row["offset"]) > thr:
			#print row[tru], row[pre], str(row["subfield"])
			ax.text(row[tru], row[pre] - row[tru], str(row["subfield"]))
	
	

sr = 0.07 # shear range
srp = 0.2 # shear range predictions
swrp = 0.02
fig = plt.figure(figsize=(22, 13))


ax = fig.add_subplot(3, 4, 1)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s1", -sr, sr), Feature("pre_s1_res", -srp, srp), Feature("psf_adamom_sigma"), yisres=True, metrics=True, showidline=True, idlinekwargs={"color":"black"})
labeloutliers(ax, cat, "pre_s1", "tru_s1")
ax.set_title("Unweighted, wide")
ax = fig.add_subplot(3, 4, 2)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s2", -sr, sr), Feature("pre_s2_res", -srp, srp), Feature("psf_adamom_sigma"), yisres=True, metrics=True, showidline=True, idlinekwargs={"color":"black"})
labeloutliers(ax, cat, "pre_s2", "tru_s2")

ax = fig.add_subplot(3, 4, 3)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s1", -sr, sr), Feature("pre_s1_res", -srp, srp), Feature("psf_adamom_g1"), yisres=True, showidline=True, idlinekwargs={"color":"black"})

ax = fig.add_subplot(3, 4, 4)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s2", -sr, sr), Feature("pre_s2_res", -srp, srp), Feature("psf_adamom_g2"), yisres=True, showidline=True, idlinekwargs={"color":"black"})



ax = fig.add_subplot(3, 4, 5)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s1", -sr, sr), Feature("pre_s1_res", -swrp, swrp), Feature("psf_adamom_sigma"), yisres=True, metrics=True, showidline=True, idlinekwargs={"color":"black"})
ax.set_title("Unweighted, narrow")
ax = fig.add_subplot(3, 4, 6)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s2", -sr, sr), Feature("pre_s2_res", -swrp, swrp), Feature("psf_adamom_sigma"), yisres=True, metrics=True, showidline=True, idlinekwargs={"color":"black"})

ax = fig.add_subplot(3, 4, 7)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s1", -sr, sr), Feature("pre_s1_res", -swrp, swrp), Feature("psf_adamom_g1"), yisres=True, showidline=True, idlinekwargs={"color":"black"})

ax = fig.add_subplot(3, 4, 8)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s2", -sr, sr), Feature("pre_s2_res", -swrp, swrp), Feature("psf_adamom_g2"), yisres=True, showidline=True, idlinekwargs={"color":"black"})




ax = fig.add_subplot(3, 4, 9)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s1", -sr, sr), Feature("pre_s1w_res", -swrp, swrp), Feature("psf_adamom_sigma"), yisres=True, metrics=True, showidline=True, idlinekwargs={"color":"black"})
labeloutliers(ax, cat, "pre_s1w", "tru_s1", thr=0.003)
ax.set_title("Weighted")
ax = fig.add_subplot(3, 4, 10)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s2", -sr, sr), Feature("pre_s2w_res", -swrp, swrp), Feature("psf_adamom_sigma"), yisres=True, metrics=True, showidline=True, idlinekwargs={"color":"black"})
labeloutliers(ax, cat, "pre_s2w", "tru_s2", thr=0.003)

ax = fig.add_subplot(3, 4, 11)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s1", -sr, sr), Feature("pre_s1w_res", -swrp, swrp), Feature("psf_adamom_g1"), yisres=True, showidline=True, idlinekwargs={"color":"black"})

ax = fig.add_subplot(3, 4, 12)
momentsml.plot.scatter.scatter(ax, cat, Feature("tru_s2", -sr, sr), Feature("pre_s2w_res", -swrp, swrp), Feature("psf_adamom_g2"), yisres=True, showidline=True, idlinekwargs={"color":"black"})


plt.tight_layout()

#if filepath:
#	plt.savefig(filepath)
#else:

plt.show()
plt.close(fig) # Helps releasing memory when calling in large loops.



import momentsml.meas


def default(catalog, stampsize, gain):
	"""
	Default measfct, runs on "img".
	"""	
	
	# HSM adamom
	catalog = momentsml.meas.galsim_adamom.measfct(catalog, stampsize=stampsize, variant="wider")
	catalog = momentsml.meas.adamom_calc.measfct(catalog)
	
	# And skystats
	catalog = momentsml.meas.skystats.measfct(catalog, stampsize=stampsize)
	
	# And snr
	catalog = momentsml.meas.snr.measfct(catalog, gain=gain)
	
	
	return catalog
	
	


default_groupcols = [
'adamom_flag',
'adamom_flux',
'adamom_x',
'adamom_y',
'adamom_g1',
'adamom_g2',
'adamom_sigma',
'adamom_rho4',
'adamom_logflux',
'adamom_g',
'adamom_theta',
'skystd',
'skymad',
'skymean',
'skymed',
'skystampsum',
'skyflag',
'snr'
]


#'fourier_x',
#'fourier_y',
#'fourier_adamom_flag',
#'fourier_adamom_flux',
#'fourier_adamom_x',
#'fourier_adamom_y',
#'fourier_adamom_g1',
#'fourier_adamom_g2',
#'fourier_adamom_sigma',
#'fourier_adamom_rho4',
#'sewpy_XWIN_IMAGE',
#'sewpy_YWIN_IMAGE',
#'sewpy_AWIN_IMAGE',
#'sewpy_BWIN_IMAGE',
#'sewpy_THETAWIN_IMAGE',
#'sewpy_FLUX_WIN',
#'sewpy_FLUXERR_WIN',
#'sewpy_NITER_WIN',
#'sewpy_FLAGS_WIN',
#'sewpy_FLUX_AUTO',
#'sewpy_FLUXERR_AUTO',
#'sewpy_FWHM_IMAGE',
#'sewpy_KRON_RADIUS',
#'sewpy_FLUX_RADIUS',
#'sewpy_FLUX_RADIUS_1',
#'sewpy_FLUX_RADIUS_2',
#'sewpy_FLUX_RADIUS_3',
#'sewpy_FLUX_RADIUS_4',
#'sewpy_FLUX_RADIUS_5',
#'sewpy_FLUX_RADIUS_6',
#'sewpy_FLUX_APER',
#'sewpy_FLUX_APER_1',
#'sewpy_FLUXERR_APER',
#'sewpy_FLUXERR_APER_1',
#'sewpy_BACKGROUND',
#'sewpy_FLAGS',
#'sewpy_assoc_flag',
#'sewpy_snr',


default_removecols = []



import momentsml
import astropy
import os
import config
import measfcts


cat = astropy.table.Table()

prefix = "psf_"

psfpos = (0.5 + config.psfstampsize/2.0, 0.5 + config.psfstampsize/2.0)

cat = astropy.table.Table([[psfpos[0]], [psfpos[1]]], names=('{}x'.format(prefix), '{}y'.format(prefix)))


# To measure the PSF, we attach the image:
cat.meta["img"] = momentsml.tools.imageinfo.ImageInfo(
    filepath=os.path.join(config.workdir, "psf.fits"),
    xname="{}x".format(prefix),
    yname="{}y".format(prefix),
    stampsize=config.psfstampsize,
    workdir=os.path.join(config.workdir, "psf_measworkdir"),
	pixelscale= 1.0 / config.psfoversampling
    )

# We measure the PSF. Not really needed, but serves as a check.
# There is no noise, so gain and snr are garbage.
meascat = measfcts.default(cat, stampsize=config.psfstampsize, gain=1.0)

print(meascat[0])

if abs(meascat["skystampsum"] - 1.0) > 0.001:
	raise RuntimeError("PSF is not well normalized ?")

# We do not save the measurement here, as we don't need all those fields to be repeated  in our further galaxy measurement catalogs.
  
momentsml.tools.io.writepickle(cat, os.path.join(config.workdir, "psfcat.pkl"))




import matplotlib
matplotlib.use("AGG")

import argparse

import momentsml.tools
import momentsml.learn
import momentsml
import tenbilac

import config
import numpy as np
import os
import glob
import shutil

#import plot_3_valw


import logging
logging.basicConfig(format=config.loggerformat, level=logging.DEBUG)
logger = logging.getLogger(__name__)



wtraindir = os.path.join(config.traindir, config.datasets["tw"] + "_with_" + config.datasets["tp"] + "_" + config.sconfname)
catpath = os.path.join(wtraindir, "groupmeascat_predforw.pkl")

cat = momentsml.tools.io.readpickle(catpath)

		
"""
# Now we might want to copy an existing training to start from there.
if args.startfrom is not None:
	logger.info("Copying existing training from {}...".format(args.startfrom))

	dirnames = momentsml.learn.tenbilacrun.confnames(config.weightconflist)
	for (dirname, conf) in zip(dirnames, config.weightconflist):
		newtraindirpath = os.path.join(traindir, dirname)
		if os.path.isdir(newtraindirpath):
			raise RuntimeError("A training already exists, but you want me to start from another one")
		
		#pretrainminipaths = sorted(glob.glob(config.great3.subpath(args.startfrom, "ml", config.datasets["train-weight"], dirname, "mini_*")))
		#if len(pretrainminipaths) == 0:
		#	raise RuntimeError("Could not find pre-training!")
		#pretrainpath = pretrainminipaths[-1] # Taking the last one
		
		# Let's not take mini, but the full one
		pretrainpath = config.great3.subpath(args.startfrom, "ml", config.datasets["train-weight"], dirname)
		logger.info("Copying {} into {}...".format(pretrainpath, newtraindirpath))
		shutil.copytree(pretrainpath, newtraindirpath)
		print "##############################   Is it ok to keep the normer ?"
"""

	
# Running the training	
dirnames = momentsml.learn.tenbilacrun.train(cat, config.weightconflist, wtraindir)

print dirnames

#dirnames = ["ada4s1w_sum33wmassshort", "ada4s2w_sum33wmassshort"]

"""
# Summary plot and self-predictions
for dirname in dirnames:
	tenbilacdirpath = os.path.join(traindir, dirname)
	ten = tenbilac.com.Tenbilac(tenbilacdirpath)
	ten._readmembers()
	tenbilac.plot.summaryerrevo(ten.committee, filepath=os.path.join(traindir, "{}_summary.png".format(dirname)))


	cat = momentsml.tools.io.readpickle(catpath) # To get the full catalog with all cases	
	
	for (dirname, conf) in zip(dirnames, config.weightconflist):

		predcat = momentsml.learn.tenbilacrun.predict(cat, [conf], traindir)
		predcatpath = os.path.join(traindir, "{}_selfpred.pkl".format(dirname))
		momentsml.tools.io.writepickle(predcat, predcatpath)
		figpredcatpath = os.path.join(traindir, "{}_selfpred.png".format(dirname))
		
		if "s2" in conf[0] or "g2" in conf[0]:
			component = 2
		else:
			component = 1
		plot_3_valw.plot(predcat, component, filepath=figpredcatpath)
			
"""



import matplotlib
matplotlib.use("AGG")

import os
import momentsml
import config
import argparse

import simparams
import measfcts
import numpy as np

import logging
logging.basicConfig(format=config.loggerformat, level=logging.DEBUG)
logger = logging.getLogger(__name__)


def define_parser():
	"""Defines the command line arguments
	"""
	parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
	parser.add_argument('code', type=str, help='what dataset should be simulated?')
	
	return parser


def configure(args):
	"""Configures settings for the different datasets
	"""
	
	code = args.code
	
	#####
	#####  Datasets for the fixed-PSF experiement (Section 6)
	#####
	
	if code == "si-1": # Just to quickly draw 1000 galaxies probing the distributions.
		sp = simparams.Fiducial(
			name = code,
			snc_type = 0,
			shear = 0,
			min_tru_sb = 1.0,
			noise_level = 1.0
		)
		drawconf = {
			"n":1000,
			"nc":10,
			"nrea":1,
			"ncat":1,
			"ncpu":1,
			"groupmode":None,
			"skipdone":False	
		}

	elif code == "tp-1": # 8 M
		sp = simparams.Fiducial_statshear(
			name = code,
			snc_type = 2000,
			shear = 0.1,
			noise_level = 1.0,
			min_tru_sb = 1.0,
		)
		drawconf = {
			"n":1,
			"nc":1,
			"nrea":1,
			"ncat":4000,
			"ncpu":10,
			"groupmode":"shear",
			"skipdone":False	
		}


	elif code == "vp-1": # 5000 cases, 10'000 SNC rotations each (50 M)
		sp = simparams.Fiducial_statshear(
			name = code,
			snc_type = 10000,
			shear = 0.1,
			noise_level = 1.0,
			min_tru_sb = 1.0,
		)
		drawconf = {
			"n":1,
			"nc":1,
			"nrea":1,
			"ncat":5000,
			"ncpu":10,
			"groupmode":"shear",
			"skipdone":False	
		}

	elif code == "tw-1": # 20 M
		sp = simparams.Fiducial_statshear(
			name = code,
			snc_type = 0,
			shear = 0.1,
			noise_level = 1.0,
			min_tru_sb = 1.0
		)
		drawconf = {
			"n":100000,
			"nc":1000,
			"nrea":1,
			"ncat":200,
			"ncpu":10,
			"groupmode":"shear",
			"skipdone":False	
		}
	


	elif code == "vo-1": # 20 M
		sp = simparams.Fiducial_statshear(
			name = code,
			snc_type = 0,
			shear = 0.1,
			noise_level = 1.0,
			min_tru_sb = 1.0
		)
		drawconf = {
			"n":100000,
			"nc":100,
			"nrea":1,
			"ncat":200,
			"ncpu":10,
			"groupmode":"shear",
			"skipdone":False	
		}

	
	#####
	#####  Datasets for the variable-PSF experiement (Section 7)
	#####



	elif code == "si-varpsf-1": # Just to quickly draw 1000 galaxies probing the distributions.
		sp = simparams.Fiducial(
			name = code,
			snc_type = 0,
			shear = 0,
			noise_level = 1.0,
			varpsf_type = 1
		)
		drawconf = {
			"n":1000,
			"nc":10,
			"nrea":1,
			"ncat":1,
			"ncpu":1,
			"groupmode":None,
			"skipdone":False	
		}
	

	elif code == "tp-varpsf-1": # similar to tp-2-faint, but with variable psf (10 M)
		sp = simparams.Fiducial_statshear(
			name = code,
			snc_type = 2000,
			shear = 0.1,
			noise_level = 1.0,
			min_tru_sb = 1.0,
			varpsf_type = 1,
		)
		drawconf = {
			"n":1,
			"nc":1,
			"nrea":1,
			"ncat":5000,
			"ncpu":25,
			"groupmode":"shear",
			"skipdone":False	
		}

	elif code == "tp-varpsf-1-pretrain": # ellipticity and low-noise pretraining (1M)
		sp = simparams.Fiducial(
			name = code,
			snc_type = 0, # No SNC
			shear = -1, # No shear here
			noise_level = 0.1,
			min_tru_sb = 1.0,
			varpsf_type = 1,
		)
		drawconf = {
			"n":10000,
			"nc":100,
			"nrea":100,
			"ncat":1,
			"ncpu":25,
			"groupmode":"ellipticity",
			"skipdone":False	
		}


	elif code == "vp-varpsf-1": # 50 M, with variable PSF
		sp = simparams.Fiducial_statshear(
			name = code,
			snc_type = 10000,
			shear = 0.1,
			noise_level = 1.0,
			min_tru_sb = 1.0,
			varpsf_type = 1,
		)
		drawconf = {
			"n":1,
			"nc":1,
			"nrea":1,
			"ncat":5000,
			"ncpu":50,
			"groupmode":"shear",
			"skipdone":False	
		}


	elif code == "tw-varpsf-1": # 1000 cases, 10'000 gals (with 2 fold SNC) (10 M)
		sp = simparams.Fiducial_statshear(
			name = code,
			snc_type = 2,
			shear = 0.1,
			noise_level = 1.0,
			min_tru_sb = 1.0,
			varpsf_type = 1,
		)
		drawconf = {
			"n":5000,
			"nc":100,
			"nrea":1,
			"ncat":1000,
			"ncpu":50,
			"groupmode":"shear",
			"skipdone":False	
		}

	elif code == "vo-varpsf-1":
		sp = simparams.Fiducial_statshear(
			name = code,
			snc_type = 2,
			shear = 0.1,
			noise_level = 1.0,
			min_tru_sb = 1.0,
			varpsf_type = 1,
		)
		drawconf = {
			"n":5000,
			"nc":100,
			"nrea":1,
			"ncat":2000,
			"ncpu":20,
			"groupmode":"shear",
			"skipdone":False	
		}

	
	else:
		
		logger.info("Unknown code '{}'".format(code))
		exit()
		
	
	return (sp, drawconf)








def run(configuration):
	"""Draws the simulations and measures them
	"""
	sp, drawconf = configuration
	
	simdir = config.simdir
	measdir = config.simmeasdir


	# Simulating images
	momentsml.sim.run.multi(
		simdir=simdir,
		simparams=sp,
		drawcatkwargs={"n":drawconf["n"], "nc":drawconf["nc"], "stampsize":config.drawstampsize},
		drawimgkwargs={}, 
		psfcat=None, psfselect="random",
		ncat=drawconf["ncat"], nrea=drawconf["nrea"], ncpu=drawconf["ncpu"],
		savepsfimg=False, savetrugalimg=False
	)


	# Measuring the newly drawn images
	momentsml.meas.run.onsims(
		simdir=simdir,
		simparams=sp,
		measdir=measdir,
		measfct=measfcts.default,
		measfctkwargs={"stampsize":config.stampsize},
		ncpu=drawconf["ncpu"],
		skipdone=drawconf["skipdone"]
	)
	

	cat = momentsml.meas.avg.onsims(
		measdir=measdir, 
		simparams=sp,
		task="group",
		groupcols=measfcts.default_groupcols, 
		removecols=measfcts.default_removecols
	)

	momentsml.tools.table.keepunique(cat)
	momentsml.tools.io.writepickle(cat, os.path.join(measdir, sp.name, "groupmeascat.pkl"))
	
	
	# Now we perform some additional computations on this catalog.
	cat = momentsml.tools.io.readpickle(os.path.join(measdir, sp.name, "groupmeascat.pkl"))
	
	if drawconf["groupmode"] == "shear":
	
		#cat = momentsml.tools.table.groupreshape(cat, groupcolnames=["tru_s1", "tru_s2"])
		cat = momentsml.tools.table.fastgroupreshape(cat, groupcolnames=["tru_s1", "tru_s2"])	
	
		momentsml.tools.table.keepunique(cat)
	
	if drawconf["groupmode"] in ["shear", "ellipticity"]:
	
		# For each case, we add the fraction of failed measurements
		nrea = cat["adamom_g1"].shape[1]
		logger.info("We have {} realizations".format(nrea))
		cat["adamom_failfrac"] = np.sum(cat["adamom_g1"].mask, axis=1) / float(nrea)		
	
		#print momentsml.tools.table.info(cat)
		momentsml.tools.io.writepickle(cat, os.path.join(measdir, sp.name, "groupmeascat.pkl"))



if __name__ == '__main__':
	parser = define_parser()
	args = parser.parse_args()
	status = run(configure(args))
	exit(status)





import os
import momentsml
import numpy as np

import config

import logging
logger = logging.getLogger(__name__)


#incatname = "vo-2"
#incatname = "tw-1"
#incatname = "tw-1-snc"
#incatname = "tw-1"
#incatname = "tw-2"
incatname = "vo-3"


#variant = "snr" # No longer used
variant = "snrsigma" #<----
#variant = "realcut" # <-----

incatpath = os.path.join(config.simmeasdir, incatname, "groupmeascat.pkl")
cat = momentsml.tools.io.readpickle(incatpath)
logger.info("Fraction of masked gals in input catalog:")
logger.info(float(np.sum(cat["adamom_sigma"].mask)) / np.size(cat["adamom_sigma"].mask))

#print momentsml.tools.table.info(cat)

if variant is "snr":

	outcatname = incatname + "-snr-above-10"
	newmask = cat["snr"] < 10.0

elif variant is "snrsigma":

	outcatname = incatname + "-snr-above-10-sigma-above-1.5"
	newmask = np.logical_or(cat["snr"] < 10.0, cat["adamom_sigma"] < 1.5)

elif variant is "realcut":

	outcatname = incatname + "-realcut"
	
	# 2 points:
	a = (24.8, 2.0)
	b = (23.0, 10.0)
	# equation y = mx + h
	m = (a[1] - b[1]) / (a[0] - b[0])
	h = a[1] - m*a[0]
	newmask = np.logical_or(cat["tru_rad"] < 1.4, cat["tru_rad"] > m*cat["tru_mag"] + h)

else:
	raise RuntimeError("Unknown")

logger.info("Fraction of selected gals:")
logger.info(float(np.sum(newmask)) / np.size(newmask) )


origmask = cat["adamom_sigma"].mask
combimask = np.logical_or(newmask, origmask)
cat["adamom_sigma"].mask = combimask


logger.info("Fraction of masked gals in output catalog:")
logger.info(float(np.sum(cat["adamom_sigma"].mask)) / np.size(cat["adamom_sigma"].mask))

outcatdir = os.path.join(config.simmeasdir, outcatname)
if not os.path.exists(outcatdir):
	os.mkdir(outcatdir)

momentsml.tools.io.writepickle(cat, os.path.join(outcatdir, "groupmeascat.pkl"))



"""
Experimental plots

"""

import logging
logger = logging.getLogger(__name__)

import matplotlib.pyplot as plt
import matplotlib.cm
from mpl_toolkits.axes_grid1 import make_axes_locatable
from matplotlib.ticker import MaxNLocator
from matplotlib.ticker import AutoMinorLocator






"""
Histogram helper functions
"""

import utils
from .. import tools

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.cm
from mpl_toolkits.axes_grid1 import make_axes_locatable
from matplotlib.ticker import MaxNLocator
from matplotlib.ticker import AutoMinorLocator

from scipy.stats import norm


import logging
logger = logging.getLogger(__name__)


def hist(ax, cat, feat, text=None, title=None, **kwargs):
	"""
	Adds a simple histogram to the axes and sets limits and label.
	This function can be called repeatedly on the same axes to overplot several catalogs,
	or say different selections from a same catalog.
	
	:param ax: a matplotlib.axes.Axes object
	:param cat: an astropy table 
	:param feat: a Feature object that I should bin
	:param text: some text to be written in the figure (top left corner)
		As we frequently want to do this, here is a simple way to do it.
		For more complicated things, add the text yourself to the axes.
	:param title: the title to place on top of the axis.
		This is here only because scatter.scatter() has the same option.

	Any further kwargs are either passed to ``hist()``.
	"""
	
	logger.debug("Preparing hist of '%s'" % (feat.colname))
	
	data = tools.feature.get1Ddata(cat, [feat], keepmasked=False)
	
	# By default, we want to limit the "binning" of the actual histogram (not just their display) to the specified range.
	# However, this fails when the "low" or "high" are set to None. Hence some explicit code:
	if feat.low is not None and feat.high is not None: 
		histrange = (feat.low, feat.high)
	else:
		histrange = None
	# If you do not like this default behaviour, you can overwrite it using the kwarg "range=None" !
	
	
	mykwargs = {"histtype":"stepfilled", "bins":100, "range":histrange, "alpha":0.5, "ec":"none", "color":"black"}
	mykwargs.update(kwargs)

	# We call hist:
	n, bins, patches = ax.hist(data[feat.colname], **mykwargs)
	
	# Set lim and label:
	if feat.low is not None and feat.high is not None:
		ax.set_xlim(feat.low, feat.high)
	ax.set_xlabel(feat.nicename)
	
	# We want minor ticks:
	ax.xaxis.set_minor_locator(AutoMinorLocator(5))
	ax.yaxis.set_minor_locator(AutoMinorLocator(5))

	# The text:
	if text:
		ax.annotate(text, xy=(0.0, 1.0), xycoords='axes fraction', xytext=(8, -8), textcoords='offset points', ha='left', va='top')
	if title:
		ax.set_title(title)


	return n, bins, patches


def errhist(ax, cat, prefeat, trufeat, normrad=3.0, **kwargs):
	"""
	Shows the distribution of prediction errors (prefeat - trufeat), as a histogram.
	If prefeat has an errcolname, each term is normalized by this errobar.
	
	"""
	
	if trufeat.errcolname is not None:
		logger.warning("The 'truth' has an uncertainty, this is strange. Inverted arguments?")
	
	data = utils.getdata(cat, [prefeat, trufeat])
	# Good to do this here (and not leave it only to hist), so that data has a reduced number
	# of columns. Indeed we add some columns below.
	
	if prefeat.errcolname is None:
		# Then we plot a simple histogram of the prediction errors
		
		data["err"] = data[prefeat.colname] - data[trufeat.colname]
		
		err = tools.feature.Feature("err", nicename = "%s - %s" % (prefeat.nicename, trufeat.nicename))
		hist(ax, data, err, normed=True, **kwargs)
	
	else:
		# We normalize the residuals with the uncertainties
		
		data["err"] = (data[prefeat.colname] - data[trufeat.colname]) / data[prefeat.errcolname]
		
		err = tools.feature.Feature("err", -normrad, normrad, nicename = "(%s - %s) / %s" % (prefeat.colname, trufeat.colname, prefeat.errcolname))
		
	
		hist(ax, data, err, normed=True, label = "Residuals normalized by uncertainty", **kwargs)
		
		x = np.linspace(-normrad, normrad, 1000)
		ax.plot(x, norm.pdf(x, 0.0, 1.0), color="black", label="Standard normal distribution")

	
	



"""
To be able to create a single training set for all branches,
we stack the starfield images into one single FITS file,
and create a single star catalog.
"""

import numpy as np
import astropy.table

import momentsml
import momentsmlgreat3
import config

import logging
logging.basicConfig(format=config.loggerformat, level=logging.DEBUG)
logger = logging.getLogger(__name__)

subfields = []
psf_adamom_g1s = []
psf_adamom_g2s = []
psf_adamom_rho4s = []
psf_adamom_sigmas = []

for (i, subfield) in enumerate(config.great3.subfields):

	catpath = config.great3.subpath(subfield, "obs", "star_meascat.pkl")
	cat = momentsml.tools.io.readpickle(catpath)
	#print momentsml.tools.table.info(cat)
	
	subfields.append(subfield)
	psf_adamom_g1s.append(np.mean(cat["psf_adamom_g1"])) # measurements of 9 stars
	psf_adamom_g2s.append(np.mean(cat["psf_adamom_g2"]))
	psf_adamom_rho4s.append(np.mean(cat["psf_adamom_rho4"]))
	psf_adamom_sigmas.append(np.mean(cat["psf_adamom_sigma"]))


# We make a catalog out of this

cat = astropy.table.Table([
		subfields, psf_adamom_g1s, psf_adamom_g2s, psf_adamom_rho4s, psf_adamom_sigmas
	], names=(
		'subfield', "psf_adamom_g1", "psf_adamom_g2", "psf_adamom_rho4", "psf_adamom_sigma"
		)
	)

cat.sort("psf_adamom_sigma")

print cat





"""
Helper functions to create scatter-like plots from an astropy table and Feature objects.
"""

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.cm
from mpl_toolkits.axes_grid1 import make_axes_locatable
from matplotlib.ticker import MaxNLocator
from matplotlib.ticker import AutoMinorLocator
from matplotlib.lines import Line2D

from .. import tools

import astropy

import logging
logger = logging.getLogger(__name__)





def scatter(ax, cat, featx, featy, featc=None, cmap="jet", title=None, text=None, showidline=False, idlinekwargs=None,
	metrics=False, sidehists=False, sidehistkwargs=None, errorbarkwargs=None, yisres=False, hidecbar=False, **kwargs):
	"""
	A simple scatter plot of cat, between two Features. A third Feature, featc, gives an optional colorbar.
	
	.. note:: If you specify this ``featc``, this function uses matplotlib's ``scatter()``. Otherwise, the function uses ``plot()``, as
		plot is much faster for large numbers of points! The possible ``**kwargs`` change accordingly!

	:param ax: a matplotlib.axes.Axes object
	:param cat: an astropy table 
	:param featx: a Feature object telling me what to draw on my x axis
	:param featy: idem for y
	:param featc: a Feature to use for the colorbar, decides if plot() or scatter() is used.
	:param cmap: the color bar to use. For a scatter plot one usually wants to see every point, avoid white!
	:param title: the title to place on top of the axis.
		The reason why we do not leave this to the user is that the placement changes when sidehists is True.
	:param text: some text to be written in the figure (top left corner)
		As we frequently want to do this, here is a simple way to do it.
		For more complicated things, add the text yourself to the axes.
	:param showidline: draws an "identity" diagonal line
	:param idlinekwargs: a dict of kwargs that will be passed to plot() to draw the idline
	:param metrics: if True, assumes that featx is a label ("tru") and featy is the corresponding predlabel ("pre"), and
		writes the RMSD and other metrics on the plot.
	:param sidehists: adds projection histograms on the top and the left (not nicely compatible with the colorbar)
		The range of these hists are limited by your features limits. Bins outside your limits are not computed!
	:param sidehistkwargs: a dict of keyword arguments to be passed to these histograms.
		Add range=None to these if you want all bins to be computed.
	:param errorbarkwargs: a dict of keywords to be passed to errorbar()
	:param yisres: if True, featy will be treated as residues when computing the metrics, and idline will be horizontal.
	
	:param hidecbar: if True, the colorbar is not drawn.
	
	Any further kwargs are either passed to ``plot()`` (if no featc is given) or to ``scatter()``.
	
	Some commonly used kwargs for plot() are:
	
	* **marker**: default is ".", you can switch to single pixel (",") or anything else...
	* **ms**: marker size in points
	* **color**: e.g. "red"
	* **label**: for the legend

	Some commonly used kwargs for scatter() are:
	
	* **s**: marker size
	* **label**: for the legend

	By default plots will be rasterized if the catalog has more than 5000 entries. To overwrite,
	just pass rasterized = True or False as kwarg.

	"""
	
	# Some initial settings:
	if sidehistkwargs is None:
		sidehistkwargs = {}
	if errorbarkwargs is None:
		errorbarkwargs = {}
	
	if len(cat) > 5000: # We rasterize plot() and scatter(), to avoid millions of vector points.
		logger.info("Plot will be rasterized, use kwarg rasterized=False if you want to avoid this")
		rasterized = True
	else:
		rasterized = False
	
	logger.info("Preparing scatter plot of '%s' against '%s'" % (featx.colname, featy.colname))
	
	# Getting the data 
	features = [featx, featy]
	if featc is not None:
		features.append(featc)
	errxfeat = featx.err()
	erryfeat = featy.err()
	if errxfeat != None:
		features.append(errxfeat)
	if erryfeat != None:
		features.append(erryfeat)
		
	#logger.warning("Should include error bars here, use featx.err... etc")
	
	data = tools.feature.get1Ddata(cat, features, keepmasked=False)
		
	# Preparing errorbars
	xerr = None
	yerr = None
	if featx.errcolname != None:
		xerr = data[featx.errcolname]
	if featy.errcolname != None:
		yerr = data[featy.errcolname]
	
	
	# And now, two options:
	if featc is not None: # We will use scatter(), to have a colorbar
		
		logger.info("Drawing %i points, with colorbar (using 'scatter')" % (len(data[featx.colname]))) # Log it as this might be slow
		
		# We prepare to use scatter, with a colorbar
		cmap = matplotlib.cm.get_cmap(cmap)
		mykwargs = {"marker":"o", "lw":0, "s":15, "cmap":cmap, "vmin":featc.low, "vmax":featc.high, "rasterized":rasterized}
		
		# We overwrite these mykwargs with any user-specified kwargs:
		mykwargs.update(kwargs)
		
		# And make the plot:
		
		if featx.errcolname != None or featy.errcolname != None:
			myerrorbarkwargs = {"fmt":"none", "capthick":0, "ecolor":"gray", "zorder":-100, "rasterized":rasterized}
			myerrorbarkwargs.update(errorbarkwargs)	
			ax.errorbar(data[featx.colname], data[featy.colname], xerr=xerr, yerr=yerr, **myerrorbarkwargs)
		
		stuff = ax.scatter(data[featx.colname], data[featy.colname], c=data[featc.colname], **mykwargs)

		if hidecbar is False:
			divider = make_axes_locatable(ax)
			cax = divider.append_axes("right", "5%", pad="3%")
			cax = plt.colorbar(stuff, cax)
			cax.set_label(featc.nicename)
		else:
			cax = None

			
	else: # We will use plot()
	
		logger.info("Drawing %i points without colorbar (using 'plot')" % (len(data[featx.colname])))
		mykwargs = {"marker":".", "ms":5, "color":"black", "ls":"None", "alpha":0.3, "rasterized":rasterized}
	
		# We overwrite these mykwargs with any user-specified kwargs:
		mykwargs.update(kwargs)
		
		# And we also prepare any errorbarkwargs
		myerrorbarkwargs = {"capthick":0, "zorder":-100, "rasterized":rasterized} # Different from the defaults for scatter() !
		myerrorbarkwargs.update(errorbarkwargs)
		
		# And now the actual plot:
		if featx.errcolname == None and featy.errcolname == None:
			# Plain plot:
			ax.plot(data[featx.colname], data[featy.colname], **mykwargs)
		else:
			mykwargs.update(myerrorbarkwargs)
			ax.errorbar(data[featx.colname], data[featy.colname], xerr=xerr, yerr=yerr, **mykwargs)
		
	
	# We want minor ticks:
	ax.xaxis.set_minor_locator(AutoMinorLocator(5))
	ax.yaxis.set_minor_locator(AutoMinorLocator(5))
	
	if sidehists:
		
		# By default, we want to limit the "binning" of the actual histograms (not just their display) to the specified ranges.
		# However, this needs some special treatment for the case when the "low" or "high" are set to None.
		if featx.low is not None and featx.high is not None: 
			histxrange = (featx.low, featx.high)
		else:
			histxrange = None
		if featy.low is not None and featy.high is not None: 
			histyrange = (featy.low, featy.high)
		else:
			histyrange = None
		# If you do not like this behaviour, simply set the sidehistkwarg "range" to None !
		
		
		# Same as for kwargs: we first define some defaults, and then update these defaults:
		
		mysidehistxkwargs = {"histtype":"stepfilled", "bins":100, "ec":"none", "color":"gray", "range":histxrange}
		mysidehistxkwargs.update(sidehistkwargs)
		mysidehistykwargs = {"histtype":"stepfilled", "bins":100, "ec":"none", "color":"gray", "range":histyrange}
		mysidehistykwargs.update(sidehistkwargs)
		
		# We prepare the axes for the hists:
		divider = make_axes_locatable(ax)
		axhistx = divider.append_axes("top", 1.0, pad=0.1, sharex=ax)
		axhisty = divider.append_axes("right", 1.0, pad=0.1, sharey=ax)
		
		# And draw the histograms		
		axhistx.hist(data[featx.colname], **mysidehistxkwargs)
		axhisty.hist(data[featy.colname], orientation='horizontal', **mysidehistykwargs)
		
		# Hiding the ticklabels
		for tl in axhistx.get_xticklabels():
			tl.set_visible(False)
		for tl in axhisty.get_yticklabels():
			tl.set_visible(False)
		
		# Reducing the number of major ticks...
		#axhistx.yaxis.set_major_locator(MaxNLocator(nbins = 2))
		#axhisty.xaxis.set_major_locator(MaxNLocator(nbins = 2))
		# Or hide them completely
		axhistx.yaxis.set_ticks([]) # or set_ticklabels([])
		axhisty.xaxis.set_ticks([])
	
		if title:
			axhistx.set_title(title)
		
	else:
		if title:
			ax.set_title(title)
	
	if showidline: # Show the "diagonal" identity line
	
	
		if idlinekwargs == None:
			idlinekwargs = {}
		myidlinekwargs = {"ls":"--", "color":"gray", "lw":1}
		myidlinekwargs.update(idlinekwargs)	
	
		if yisres: # Then the line is horizontal at zero
			ax.axhline(y=0, **myidlinekwargs)
			
		else: # We have to compute the endpoints
	
			# It would be nice to get this working with less code
			# (usign get_lims and axes transforms, for instance)
			# But in the meantime this seems to work fine.
			# It has to be so complicated to keep the automatic ranges working if low and high are None !
		
			# For "low":
			if featx.low is None or featy.low is None: # We use the data...
				minid = max(np.min(data[featx.colname]), np.min(data[featy.colname]))
			else:
				minid = max(featx.low, featy.low)
			# Same for "high":
			if featx.high is None or featy.high is None: # We use the data...
				maxid = min(np.max(data[featx.colname]), np.max(data[featy.colname]))
			else:
				maxid = min(featx.high, featy.high)
				
			# And we plot the line:
			ax.plot((minid, maxid), (minid, maxid), **myidlinekwargs)


	ax.set_xlim(featx.low, featx.high)
	ax.set_ylim(featy.low, featy.high)
	ax.set_xlabel(featx.nicename)
	ax.set_ylabel(featy.nicename)


	# Finally, we write the text:
	if text:
		ax.annotate(text, xy=(0.0, 1.0), xycoords='axes fraction', xytext=(8, -8), textcoords='offset points', ha='left', va='top')
	
	if metrics:
		"""
		metrics_label = featx.colname
		metrics_predlabel = featy.colname
		
		# Hack hack hack
		label_data = cat[metrics_label]
		predlabel_data = cat[metrics_predlabel][:,0]
		
		minicat = astropy.table.Table((label_data, predlabel_data))
		"""
		logger.info("Now computing metrics for this scatter plot...")
		try:
			#metrics = tools.metrics.metrics(cat, metrics_label, metrics_predlabel)
			metrics = tools.metrics.metrics(cat, featx, featy, pre_is_res=yisres)
			
			metrics_text = "predfrac: %.3f\nRMSD: %.5f\nm*1e3: %.1f +/- %.1f\nc*1e3: %.1f +/- %.1f" % (metrics["predfrac"], metrics["rmsd"], metrics["m"]*1000.0, metrics["merr"]*1000.0, metrics["c"]*1000.0, metrics["cerr"]*1000.0)
			ax.annotate(metrics_text, xy=(0.0, 1.0), xycoords='axes fraction', xytext=(8, -22), textcoords='offset points', ha='left', va='top')
		except:
			logger.warning("Metrics compuation failed", exc_info = True)
		
	if featc is not None:
		return cax


def simobs(ax, simcat, obscat, featx, featy, sidehists=True, sidehistkwargs=None, title=None, legend=False, pale=False, **kwargs):
	"""
	A scatter plot overplotting simulations (in red) and observations (in blue, like the sky).
	Previously the observations were green (like nature), but blue is better for most colorblind people.
			
	:param ax: a matplotlib Axes object
	:param simcat: simulation catalog
	:param obscat: observation catalog
	:param featx: a Feature object telling me what to draw on my x axis
	:param featy: idem for y
	:param sidehists: set this to False if you don't want side histograms
	:param sidehistkwargs: keyword arguments passed to the side hists
	:param title: the title to place on top of the axis.
		The reason why we do not leave this to the user is that the placement changes when sidehists is True.
	:param legend: if True, it writes a self-styled non-invasive "legend" in the top right corner
	:param pale: if True, points are plotted in pale colours so that contours can be overplotted.
	
	All further **kwargs** are passed to axes.plot() to make the scatter plot.
		
	"""
	
	# Could we warn the user in case it seems that the catalogs are inverted ?
	# (not implemented -- maybe by detecting the precens of some typical "sim" fields in the obscat ?)
	
	simdata = tools.feature.get1Ddata(simcat, [featx, featy], keepmasked=False)
	obsdata = tools.feature.get1Ddata(obscat, [featx, featy] , keepmasked=False)
	
	
	simcolor="red"
	obscolor="blue"
	if pale:
		simcolor_points = '#FF9999'
		obscolor_points = '#9999FF'
	else:
		simcolor_points = simcolor
		obscolor_points = obscolor
	
	if len(simcat) > 5000 or len(obscat) > 5000: # We rasterize plot() to avoid millions of vector points.
		logger.info("Plot will be rasterized, use kwarg rasterized=False if you want to avoid this")
		rasterized = True
	else:
		rasterized = False

	# First we use plot() to get a scatter, directly on the axes:
	plotkwargs = {"marker":".", "ms":5, "ls":"None", "alpha":0.3, "rasterized":rasterized}
	plotkwargs.update(kwargs)
	ax.plot(simdata[featx.colname], simdata[featy.colname], color=simcolor_points, **plotkwargs)
	ax.plot(obsdata[featx.colname], obsdata[featy.colname], color=obscolor_points, **plotkwargs)
	
	
	# Now we build the sidehists:
	if sidehists:
	
		# By default, we want to limit the "binning" of the actual histograms (not just their display) to the specified ranges.
		# However, this fails when the "low" or "high" are set to None. Hence some explicit code:
		if featx.low is not None and featx.high is not None: 
			histxrange = (featx.low, featx.high)
		else:
			histxrange = None
		if featy.low is not None and featy.high is not None: 
			histyrange = (featy.low, featy.high)
		else:
			histyrange = None
		# If you do not like this default behaviour, you can overwrite it using the sidehistkwarg "range=None" !
	
		# We now prepare the kwargs for calling hist:
		if sidehistkwargs is None:
			sidehistkwargs = {}
		mysidehistxkwargs = {"histtype":"stepfilled", "normed":"True", "bins":100, "alpha":0.5, "range":histxrange} # for x
		mysidehistxkwargs.update(sidehistkwargs)
		mysidehistykwargs = {"histtype":"stepfilled", "normed":"True", "bins":100, "alpha":0.5, "range":histyrange} # for y
		mysidehistykwargs.update(sidehistkwargs)
	
		divider = make_axes_locatable(ax)
		axhistx = divider.append_axes("top", 1.0, pad=0.1, sharex=ax)
		axhisty = divider.append_axes("right", 1.0, pad=0.1, sharey=ax)
		
		
		axhistx.hist(simdata[featx.colname], color=simcolor, ec=simcolor, **mysidehistxkwargs)
		axhistx.hist(obsdata[featx.colname], color=obscolor, ec=obscolor, **mysidehistxkwargs)
		
		
		axhisty.hist(simdata[featy.colname], color=simcolor, ec=simcolor, orientation='horizontal', **mysidehistykwargs)
		axhisty.hist(obsdata[featy.colname], color=obscolor, ec=obscolor, orientation='horizontal', **mysidehistykwargs)
		
		# Hiding the ticklabels
		for tl in axhistx.get_xticklabels():
			tl.set_visible(False)
		for tl in axhisty.get_yticklabels():
			tl.set_visible(False)

		# Hide the hist ticks
		axhistx.yaxis.set_ticks([]) # or set_ticklabels([])
		axhisty.xaxis.set_ticks([])
	
		if title:
			axhistx.set_title(title)
		
	else:
		if title:
			ax.set_title(title)

	# We set the limits and labels:
	ax.set_xlim(featx.low, featx.high)
	ax.set_ylim(featy.low, featy.high)
	ax.set_xlabel(featx.nicename)
	ax.set_ylabel(featy.nicename)
	
	# We want minor ticks:
	ax.xaxis.set_minor_locator(AutoMinorLocator(5))
	ax.yaxis.set_minor_locator(AutoMinorLocator(5))
	
	if legend:
		ax.annotate("Simulations", color=simcolor, xy=(1.0, 1.0), xycoords='axes fraction', xytext=(-8, -8), textcoords='offset points', ha='right', va='top')
		ax.annotate("Observations", color=obscolor, xy=(1.0, 1.0), xycoords='axes fraction', xytext=(-8, -24), textcoords='offset points', ha='right', va='top')
	
	
	
	



"""
Validation plot on training-like data structure, with one galaxy per case
"""

import momentsml
import momentsmlgreat3
import momentsml.plot
from momentsml.tools.feature import Feature
import matplotlib.pyplot as plt
import matplotlib.colors
import numpy as np

import config

import os
import logging
logging.basicConfig(format=config.loggerformat, level=logging.INFO)
logger = logging.getLogger(__name__)




def main():

	print "This is no longer a standalone script"



def plot(cat, component, mode="s", filepath=None, select=None, withweights=False):


	logger.info("Running on component {} with mode {}".format(component, mode))
	#cat["adamom_log_flux"] = np.log10(cat["adamom_flux"])
	#cat["adamom_frac"] = np.sum(cat["adamom_g1"].mask, axis=1)/float(cat["adamom_g1"].shape[1])
	
	prename = "pre_s{}".format(component)
	
	cat["pre_maskedfrac"] = np.sum(cat[prename].mask, axis=1) / float(cat[prename].shape[1])
	#cat["pre_frac"] = (float(cat[prename].shape[1]) - np.sum(cat[prename].mask, axis=1)) / float(cat[prename].shape[1])
	
	#b = cat["tru_s1", "pre_maskedfrac", "adamom_failfrac"]
	#b.show_in_browser()
	#exit()
	#assert np.allclose(["pre_maskedfrac"], cat["adamom_failfrac"])
	
	pre_maskedfrac = Feature("pre_maskedfrac")
	
	momentsml.tools.table.addstats(cat, "adamom_flux")
	momentsml.tools.table.addstats(cat, "adamom_sigma")
	momentsml.tools.table.addstats(cat, "snr")
	snr_mean = Feature("snr_mean")

	
	#print min(cat["adamom_frac"]), max(cat["adamom_frac"]), np.median(cat["adamom_frac"])

	if select is not None:
		cat = select.select(cat)

	#print min(cat["snr_mean"]), max(cat["snr_mean"]), np.median(cat["snr_mean"])
	
	
	momentsml.tools.table.addrmsd(cat, "pre_s{}".format(component), "tru_{}{}".format(mode, component))
	
	if withweights == False:
		momentsml.tools.table.addstats(cat, "pre_s{}".format(component))
		pre_sc_bias = Feature("pre_s{}_bias".format(component))
		pre_sc_mean = Feature("pre_s{}_mean".format(component))

	else: # Then we use weights if available)
		
		if not "pre_s{}w".format(component) in cat.colnames:
			# It doesn't make sense to fake them for this plot, just skip it!
			logger.warning("No weights available, skipping")
			return

		momentsml.tools.table.addstats(cat, "pre_s{}".format(component), wcol="pre_s{}w".format(component))
		cat["pre_s{}_wbias".format(component)] = cat["pre_s{}_wmean".format(component)] - cat["tru_{}{}".format(mode, component)]
		pre_sc_bias = Feature("pre_s{}_wbias".format(component))
		pre_sc_mean = Feature("pre_s{}_wmean".format(component))
	
	tru_sc = Feature("tru_{}{}".format(mode, component))
	

	ebarmode = "scatter"

	fig = plt.figure(figsize=(24, 12))

	ax = fig.add_subplot(3, 5, 1)
	momentsml.plot.scatter.scatter(ax, cat, tru_sc,  pre_sc_mean, featc=snr_mean, showidline=True, metrics=True)
	
	ax = fig.add_subplot(3, 5, 2)
	#momentsml.plot.scatter.scatter(ax, cat, tru_sc, Feature("snr_mean"), pre_sc_bias)
	momentsml.plot.scatter.scatter(ax, cat, tru_sc, Feature("snr", rea=1), pre_sc_bias)
	ax = fig.add_subplot(3, 5, 3)
	cnorm = matplotlib.colors.SymLogNorm(linthresh=0.01)
	momentsml.plot.scatter.scatter(ax, cat, tru_sc, Feature("tru_rad"), pre_sc_bias, norm=cnorm)
	ax = fig.add_subplot(3, 5, 4)
	momentsml.plot.scatter.scatter(ax, cat, tru_sc, Feature("tru_sersicn"), pre_sc_bias)
	ax = fig.add_subplot(3, 5, 5)
	momentsml.plot.scatter.scatter(ax, cat, tru_sc, Feature("tru_g"), pre_sc_bias)

	
	ax = fig.add_subplot(3, 5, 6)
	momentsml.plot.scatter.scatter(ax, cat, Feature("tru_flux"), pre_sc_bias, Feature("tru_rad"))
	ax = fig.add_subplot(3, 5, 7)
	#momentsml.plot.scatter.scatter(ax, cat, tru_sc,  Feature("adamom_frac"), pre_sc_bias)
	momentsml.plot.scatter.scatter(ax, cat, pre_maskedfrac, pre_sc_bias, sidehists=True)
	#ax.set_xscale("log", nonposy='clip')
	
	
	ax = fig.add_subplot(3, 5, 8)
	#momentsml.plot.scatter.scatter(ax, cat, tru_sc,  Feature("tru_flux"), pre_sc_bias, norm=cnorm)
	momentsml.plot.scatter.scatter(ax, cat, tru_sc,  Feature("tru_rad"), pre_sc_bias)
	ax = fig.add_subplot(3, 5, 9)
	#momentsml.plot.scatter.scatter(ax, cat, Feature("tru_flux"), pre_sc_bias, Feature("tru_rad"))
	#momentsml.plot.scatter.scatter(ax, cat, tru_sc,  Feature("adamom_flux_mean"), pre_gc_bias)
	momentsml.plot.scatter.scatter(ax, cat, tru_sc,  Feature("tru_flux"), pre_sc_bias)
	ax = fig.add_subplot(3, 5, 10)
	#momentsml.plot.scatter.scatter(ax, cat, tru_sc,  Feature("adamom_sigma", rea=1), pre_sc_bias)
	#momentsml.plot.scatter.scatter(ax, cat, tru_sc,  Feature("tru_sb"), pre_sc_bias)
	momentsml.plot.scatter.scatter(ax, cat, Feature("tru_flux"), Feature("pre_s{}".format(component), rea=1), Feature("tru_rad"))
	
	
	ax = fig.add_subplot(3, 5, 11)
	momentsml.plot.bin.res(ax, cat, tru_sc, pre_sc_mean, Feature("tru_sb"), ncbins=3, equalcount=True, ebarmode=ebarmode)
	ax = fig.add_subplot(3, 5, 12)
	momentsml.plot.bin.res(ax, cat, tru_sc, pre_sc_mean, Feature("tru_flux"), ncbins=3, equalcount=True, ebarmode=ebarmode)
	ax = fig.add_subplot(3, 5, 13)
	momentsml.plot.bin.res(ax, cat, tru_sc, pre_sc_mean, Feature("tru_rad"), ncbins=3, equalcount=True, ebarmode=ebarmode)
	ax = fig.add_subplot(3, 5, 14)
	momentsml.plot.bin.res(ax, cat, tru_sc, pre_sc_mean, Feature("tru_sersicn"), ncbins=3, equalcount=True, ebarmode=ebarmode)
	ax = fig.add_subplot(3, 5, 15)
	momentsml.plot.bin.res(ax, cat, tru_sc, pre_sc_mean, Feature("tru_g"), ncbins=3, equalcount=True, ebarmode=ebarmode)

	
	
	
	plt.tight_layout()

	if filepath:
		logger.info("Writing plot to '{}'...".format(filepath))
		plt.savefig(filepath)
	else:
		plt.show()
	plt.close(fig) # Helps releasing memory when calling in large loops.


if __name__ == "__main__":
    main()
	
